@@ -4,15 +4,17 @@
 
 using namespace sirius;
 using namespace sddk;
+using namespace la;
+using namespace fft;
 
 void test1()
 {
-if (Communicator::world().rank() == 0) {
+if (mpi::Communicator::world().rank() == 0) {
     std::cout << "test1" << std::endl;
 }
 /* reciprocal lattice vectors in
    inverse atomic units */
-matrix3d<double> M = {{1, 0, 0},
+r3::matrix<double> M = {{1, 0, 0},
                       {0, 1, 0},
                       {0, 0, 1}};
 /* G-vector cutoff radius in
@@ -22,7 +24,7 @@ double Gmax = 10;
    last boolean parameter switches
    off the reduction of G-vectors by 
    inversion symmetry */
-Gvec gvec(M, Gmax, Communicator::world(), false);
+Gvec gvec(M, Gmax, mpi::Communicator::world(), false);
 /* loop over local number of G-vectors
    for current MPI rank */
 for (int j = 0; j < gvec.count(); j++) {
@@ -42,12 +44,12 @@ for (int j = 0; j < gvec.count(); j++) {
 
 void test2()
 {
-if (Communicator::world().rank() == 0) {
+if (mpi::Communicator::world().rank() == 0) {
     std::cout << "test2" << std::endl;
 }
 /* reciprocal lattice vectors in 
     inverse atomic units */
-matrix3d<double> M = {{1, 0, 0},
+r3::matrix<double> M = {{1, 0, 0},
                       {0, 1, 0},
                       {0, 0, 1}};
 /* G-vector cutoff radius in
@@ -57,22 +59,22 @@ double Gmax = 10;
    last boolean parameter switches
    off the reduction of G-vectors by 
    inversion symmetry */
-Gvec gvec(M, Gmax, Communicator::world(), false);
+Gvec gvec(M, Gmax, mpi::Communicator::world(), false);
 /* FFT-friendly distribution */
-Gvec_fft gvp(gvec, Communicator::world(),
-        Communicator::self());
+Gvec_fft gvp(gvec, mpi::Communicator::world(),
+        mpi::Communicator::self());
 /* dimensions of the FFT box */
-FFT3D_grid dims({40, 40, 40});
+fft::Grid dims({40, 40, 40});
 
 /* this is how our code splits the z-dimension
  * of the FFT buffer */
-auto spl_z = split_fft_z(dims[2],
-        Communicator::world());
+auto spl_z = split_z_dimension(dims[2],
+        mpi::Communicator::world());
 
 /* create SpFFT grid object */
 spfft_grid_type<double> spfft_grid(dims[0], dims[1],
-        dims[2], gvp.zcol_count_fft(), spl_z.local_size(),
-        SPFFT_PU_HOST, -1, Communicator::world().mpi_comm(),
+        dims[2], gvp.zcol_count(), spl_z.local_size(),
+        SPFFT_PU_HOST, -1, mpi::Communicator::world().native(),
         SPFFT_EXCH_DEFAULT);
 
 auto const& gv = gvp.gvec_array();
@@ -81,14 +83,14 @@ spfft_transform_type<double> spfft(
         spfft_grid.create_transform(SPFFT_PU_HOST,
             SPFFT_TRANS_C2C,
             dims[0], dims[1], dims[2], spl_z.local_size(),
-            gvp.gvec_count_fft(),
+            gvp.count(),
             SPFFT_INDEX_TRIPLETS, gv.at(memory_t::host)));
 
 /* create data buffer with local number of G-vectors
    and fill with random numbers */
-mdarray<double_complex, 1> f(gvp.gvec_count_fft());
+mdarray<std::complex<double>, 1> f(gvp.count());
 f = [](int64_t){
-  return utils::random<double_complex>();
+  return utils::random<std::complex<double>>();
 };
 /* transform to real space */
 spfft.backward(reinterpret_cast<double const*>(
@@ -112,14 +114,14 @@ for (int j0 = 0; j0 < dims[0]; j0++) {
 
 void test3()
 {
-if (Communicator::world().rank() == 0) {
+if (mpi::Communicator::world().rank() == 0) {
     std::cout << "test3" << std::endl;
 }
 spla::Context spla_ctx(SPLA_PU_HOST);
 
 /* reciprocal lattice vectors in 
    inverse atomic units */
-matrix3d<double> M = {{1, 0, 0},
+r3::matrix<double> M = {{1, 0, 0},
                       {0, 1, 0},
                       {0, 0, 1}};
 /* G-vector cutoff radius in
@@ -129,7 +131,7 @@ double Gmax = 10;
    last boolean parameter switches
    off the reduction of G-vectors by 
    inversion symmetry */
-auto gvec = std::make_shared<Gvec>(M, Gmax, Communicator::world(), false);
+auto gvec = std::make_shared<Gvec>(M, Gmax, mpi::Communicator::world(), false);
 /* number of wave-functions */
 int N = 100;
 /* create scalar wave-functions for N bands */
@@ -138,15 +140,15 @@ wf::Wave_functions<double> wf(gvec, wf::num_mag_dims(0), wf::num_bands(N), memor
 for (int i = 0; i < N; i++) {
     for (int j = 0; j < gvec->count(); j++) {
         wf.pw_coeffs(j, wf::spin_index(0), wf::band_index(i)) =
-            utils::random<double_complex>();
+            utils::random<std::complex<double>>();
     }
 }
 /* create a 2x2 BLACS grid */
-BLACS_grid grid(Communicator::world(), 2, 2);
+BLACS_grid grid(mpi::Communicator::world(), 2, 2);
 /* cyclic block size */
 int bs = 16;
 /* create a distributed overlap matrix */
-dmatrix<double_complex> o(N, N, grid, bs, bs);
+dmatrix<std::complex<double>> o(N, N, grid, bs, bs);
 /* create temporary wave-functions */
 wf::Wave_functions<double> tmp(gvec, wf::num_mag_dims(0), wf::num_bands(N), memory_t::host);
 /* orthogonalize wave-functions */
