@@ -5,7 +5,7 @@
 typedef double gemm_type;
 int const nop_gemm = 2;
 #else
-typedef double_complex gemm_type;
+typedef std::complex<double> gemm_type;
 int const nop_gemm = 8;
 #endif
 
@@ -45,8 +45,8 @@ double test_gemm(int M, int N, int K, int transa)
     printf("c.ld() = %i\n", c.ld());
     const char ta[] = {'N', 'T', 'C'};
     double t = -utils::wtime();
-    sddk::linalg(sddk::linalg_t::blas).gemm(ta[transa], 'N', M, N, K, &sddk::linalg_const<gemm_type>::one(),
-        a.at(sddk::memory_t::host), a.ld(), b.at(sddk::memory_t::host), b.ld(), &sddk::linalg_const<gemm_type>::zero(),
+    la::wrap(la::lib_t::blas).gemm(ta[transa], 'N', M, N, K, &la::constant<gemm_type>::one(),
+        a.at(sddk::memory_t::host), a.ld(), b.at(sddk::memory_t::host), b.ld(), &la::constant<gemm_type>::zero(),
         c.at(sddk::memory_t::host), c.ld());
     t += utils::wtime();
     double perf = nop_gemm * 1e-9 * M * N * K / t;
@@ -64,16 +64,16 @@ double test_pgemm(int M, int N, int K, int nrow, int ncol, int transa, int n, in
     //== pout.printf("rank : %3i, free GPU memory (Mb) : %10.2f\n", Platform::mpi_rank(), cuda_get_free_mem() / double(1 << 20));
     //== pout.flush(0);
     //== #endif
-    sddk::BLACS_grid blacs_grid(sddk::Communicator::world(), nrow, ncol);
+    la::BLACS_grid blacs_grid(mpi::Communicator::world(), nrow, ncol);
 
-    sddk::dmatrix<gemm_type> a, b, c;
+    la::dmatrix<gemm_type> a, b, c;
     if (transa == 0) {
-        a = sddk::dmatrix<gemm_type>(nullptr, M, K, blacs_grid, bs, bs);
+        a = la::dmatrix<gemm_type>(nullptr, M, K, blacs_grid, bs, bs);
     } else {
-        a = sddk::dmatrix<gemm_type>(nullptr, K, M, blacs_grid, bs, bs);
+        a = la::dmatrix<gemm_type>(nullptr, K, M, blacs_grid, bs, bs);
     }
-    b = sddk::dmatrix<gemm_type>(nullptr, K, N, blacs_grid, bs, bs);
-    c = sddk::dmatrix<gemm_type>(nullptr, M, N - n, blacs_grid, bs, bs);
+    b = la::dmatrix<gemm_type>(nullptr, K, N, blacs_grid, bs, bs);
+    c = la::dmatrix<gemm_type>(nullptr, M, N - n, blacs_grid, bs, bs);
     a.allocate(sddk::memory_t::host);
     b.allocate(sddk::memory_t::host);
     c.allocate(sddk::memory_t::host);
@@ -92,7 +92,7 @@ double test_pgemm(int M, int N, int K, int nrow, int ncol, int transa, int n, in
 
     c.zero();
 
-    if (sddk::Communicator::world().rank() == 0) {
+    if (mpi::Communicator::world().rank() == 0) {
         printf("testing parallel gemm with M, N, K = %i, %i, %i, opA = %i\n", M, N - n, K, transa);
         printf("nrow, ncol = %i, %i, bs = %i\n", nrow, ncol, bs);
     }
@@ -100,14 +100,13 @@ double test_pgemm(int M, int N, int K, int nrow, int ncol, int transa, int n, in
     gemm_type one = 1;
     gemm_type zero = 0;
     const char TA [] = {'N', 'T', 'C'};
-    sddk::linalg(sddk::linalg_t::scalapack).gemm(TA[transa], 'N', M, N - n, K, &one, a, 0, 0, b, 0, n, &zero, c, 0, 0);
+    la::wrap(la::lib_t::scalapack).gemm(TA[transa], 'N', M, N - n, K, &one, a, 0, 0, b, 0, n, &zero, c, 0, 0);
     //== #ifdef _GPU_
     //== cuda_device_synchronize();
     //== #endif
     t += utils::wtime();
     double perf = nop_gemm * 1e-9 * M * (N - n) * K / t / nrow / ncol;
-    if (sddk::Communicator::world().rank() == 0)
-    {
+    if (mpi::Communicator::world().rank() == 0) {
         printf("execution time : %12.6f seconds\n", t);
         printf("performance    : %12.6f GFlops / rank\n", perf);
     }
@@ -219,7 +218,7 @@ int main(int argn, char **argv)
             //    perf += test_pgemm_aHa(M, K, nrow, ncol, bs);
             //}
         }
-        if (sddk::Communicator::world().rank() == 0) {
+        if (mpi::Communicator::world().rank() == 0) {
             printf("\n");
             printf("average performance    : %12.6f GFlops / rank\n", perf / repeat);
         }
