@@ -5,13 +5,13 @@ using namespace sirius;
 int main(int argn, char** argv)
 {
     sirius::initialize(1);
-    sddk::pstdout pout(sddk::Communicator::world());
-    pout << "Hello from rank : " << sddk::Communicator::world().rank() << std::endl;
+    mpi::pstdout pout(mpi::Communicator::world());
+    pout << "Hello from rank : " << mpi::Communicator::world().rank() << std::endl;
 
     /* this is a collective operation */
     auto s = pout.get().str();
 
-    if (sddk::Communicator::world().rank() == 0) {
+    if (mpi::Communicator::world().rank() == 0) {
         std::cout << s;
     }
 
