@@ -1,6 +1,7 @@
 #include <sirius.hpp>
 
 using namespace sirius;
+using namespace mpi;
 using namespace sddk;
 
 int main(int argn, char** argv)
@@ -9,11 +10,11 @@ int main(int argn, char** argv)
 
     bool pr = (Communicator::world().rank() == 0);
 
-    sddk::mdarray<double_complex, 2> tmp_d(100, 200);
-    sddk::mdarray<double_complex, 2> tmp(100, 200);
+    sddk::mdarray<std::complex<double>, 2> tmp_d(100, 200);
+    sddk::mdarray<std::complex<double>, 2> tmp(100, 200);
 
-    std::cout << Communicator::world().rank() << " tmp_d " << tmp_d.at(memory_t::host) << std::endl;
-    std::cout << Communicator::world().rank() << " tmp " << tmp.at(memory_t::host) << std::endl;
+    std::cout << Communicator::world().rank() << " tmp_d " << tmp_d.at(sddk::memory_t::host) << std::endl;
+    std::cout << Communicator::world().rank() << " tmp " << tmp.at(sddk::memory_t::host) << std::endl;
 
     if (pr) {
         std::cout << "allreduce<double> " << std::endl;
@@ -34,22 +35,22 @@ int main(int argn, char** argv)
     }
 
     if (pr) {
-        std::cout << "allreduce<reinterpret_cast<double_complex>> " << std::endl;
+        std::cout << "allreduce<reinterpret_cast<std::complex<double>>> " << std::endl;
     }
     Communicator::world().allreduce(reinterpret_cast<double*>(tmp.at(memory_t::host)), 2 * static_cast<int>(tmp.size()));
 
     if (pr) {
-        std::cout << "reduce<reinterpret_cast<double_complex>> " << std::endl;
+        std::cout << "reduce<reinterpret_cast<std::complex<double>>> " << std::endl;
     }
     Communicator::world().reduce(reinterpret_cast<double*>(tmp.at(memory_t::host)), 2 * static_cast<int>(tmp.size()), 1);
 
     if (pr) {
-        std::cout << "allreduce<double_complex> " << std::endl;
+        std::cout << "allreduce<std::complex<double>> " << std::endl;
     }
     Communicator::world().allreduce(tmp.at(memory_t::host), static_cast<int>(tmp.size()));
 
     if (pr) {
-        std::cout << "reduce<double_complex> " << std::endl;
+        std::cout << "reduce<std::complex<double>> " << std::endl;
     }
     Communicator::world().reduce(tmp.at(memory_t::host), static_cast<int>(tmp.size()), 1);
 
