@@ -4,6 +4,7 @@
 
 using namespace sirius;
 using namespace sddk;
+using namespace mpi;
 
 template <typename T>
 int test_fft_complex(cmd_args& args, device_t fft_pu__)
@@ -15,40 +16,40 @@ int test_fft_complex(cmd_args& args, device_t fft_pu__)
         eps = 1e-6;
     }
 
-    matrix3d<double> M = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
+    r3::matrix<double> M = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
 
-    auto fft_grid = get_min_fft_grid(cutoff, M);
+    auto fft_grid = fft::get_min_grid(cutoff, M);
 
-    auto spl_z = split_fft_z(fft_grid[2], Communicator::world());
+    auto spl_z = fft::split_z_dimension(fft_grid[2], Communicator::world());
 
-    Gvec gvec(M, cutoff, Communicator::world(), false);
+    fft::Gvec gvec(M, cutoff, Communicator::world(), false);
 
-    Gvec_fft gvp(gvec, Communicator::world(), Communicator::self());
+    fft::Gvec_fft gvp(gvec, Communicator::world(), Communicator::self());
 
-    spfft_grid_type<T> spfft_grid(fft_grid[0], fft_grid[1], fft_grid[2], gvp.zcol_count_fft(), spl_z.local_size(),
-                           SPFFT_PU_HOST, -1, Communicator::world().mpi_comm(), SPFFT_EXCH_DEFAULT);
+    fft::spfft_grid_type<T> spfft_grid(fft_grid[0], fft_grid[1], fft_grid[2], gvp.zcol_count(), spl_z.local_size(),
+                           SPFFT_PU_HOST, -1, Communicator::world().native(), SPFFT_EXCH_DEFAULT);
 
     const auto fft_type = gvec.reduced() ? SPFFT_TRANS_R2C : SPFFT_TRANS_C2C;
 
     auto const& gv = gvp.gvec_array();
-     spfft_transform_type<T> spfft(spfft_grid.create_transform(SPFFT_PU_HOST, fft_type, fft_grid[0], fft_grid[1], fft_grid[2],
-        spl_z.local_size(), gvp.gvec_count_fft(), SPFFT_INDEX_TRIPLETS,
+    fft::spfft_transform_type<T> spfft(spfft_grid.create_transform(SPFFT_PU_HOST, fft_type, fft_grid[0], fft_grid[1], fft_grid[2],
+        spl_z.local_size(), gvp.count(), SPFFT_INDEX_TRIPLETS,
         gv.at(memory_t::host)));
 
-    sddk::mdarray<std::complex<T>, 1> f(gvp.gvec_count_fft());
-    for (int ig = 0; ig < gvp.gvec_count_fft(); ig++) {
+    sddk::mdarray<std::complex<T>, 1> f(gvp.count());
+    for (int ig = 0; ig < gvp.count(); ig++) {
         f[ig] = utils::random<std::complex<T>>();
     }
-    sddk::mdarray<std::complex<T>, 1> g(gvp.gvec_count_fft());
+    sddk::mdarray<std::complex<T>, 1> g(gvp.count());
 
     spfft.backward(reinterpret_cast<T const*>(&f[0]), spfft.processing_unit());
     spfft.forward(spfft.processing_unit(), reinterpret_cast<T*>(&g[0]), SPFFT_FULL_SCALING);
 
     double diff{0};
-    for (int ig = 0; ig < gvp.gvec_count_fft(); ig++) {
+    for (int ig = 0; ig < gvp.count(); ig++) {
         diff = std::max(diff, static_cast<double>(std::abs(f[ig] - g[ig])));
     }
-    Communicator::world().allreduce<double, sddk::mpi_op_t::max>(&diff, 1);
+    Communicator::world().allreduce<double, mpi::op_t::max>(&diff, 1);
 
     if (diff > eps) {
         return 1;
