@@ -10,7 +10,7 @@ int test1()
 
     SHT sht(device_t::CPU, lmax);
 
-    mdarray<double_complex, 2> ylm(utils::lmmax(lmax), sht.num_points());
+    mdarray<std::complex<double>, 2> ylm(utils::lmmax(lmax), sht.num_points());
     for (int i = 0; i < sht.num_points(); i++) {
         sf::spherical_harmonics(lmax, sht.theta(i), sht.phi(i), &ylm(0, i));
     }
@@ -23,7 +23,7 @@ int test1()
                 for (int m2 = -l2; m2 <= l2; m2++) {
                     for (int l3 = 0; l3 <= 8; l3++) {
                         for (int m3 = -l3; m3 <= l3; m3++) {
-                            double_complex s{0};
+                            std::complex<double> s{0};
                             for (int i = 0; i < sht.num_points(); i++) {
                                 s += std::conj(ylm(utils::lm(l1, m1), i)) * ylm(utils::lm(l2, m2), i) *
                                      ylm(utils::lm(l3, m3), i) * sht.weight(i);
