@@ -4,19 +4,19 @@ using namespace sirius;
 
 int run_test()
 {
-    vector3d<double> a(1.1, 2.2, 3.3);
-    vector3d<double> b = a;
-    vector3d<double> c(b);
-    vector3d<double> d;
+    r3::vector<double> a(1.1, 2.2, 3.3);
+    r3::vector<double> b = a;
+    r3::vector<double> c(b);
+    r3::vector<double> d;
     d = c;
-    matrix3d<int> R = {{1,0,0},{0,1,0},{0,0,1}};
+    r3::matrix<int> R = {{1,0,0},{0,1,0},{0,0,1}};
     auto e = dot(R, a) + b;
-    vector3d<double> ref(2.2, 4.4, 6.6);
+    r3::vector<double> ref(2.2, 4.4, 6.6);
     if ((ref - e).length() > 1e-16) {
         return 1;
     }
-    auto x = a + vector3d<int>(4, 5, 6);
-    vector3d<double> ref1(5.1, 7.2, 9.3);
+    auto x = a + r3::vector<int>(4, 5, 6);
+    r3::vector<double> ref1(5.1, 7.2, 9.3);
     if ((ref1 - x).length() > 1e-16) {
         return 2;
     }
