@@ -4,22 +4,21 @@
 /* test G-vectors */
 
 using namespace sirius;
-using namespace sddk;
 
 int run_test(cmd_args& args)
 {
-    std::vector<int> vd = args.value("dims", std::vector<int>({132, 132, 132}));
-    vector3d<int> dims(vd[0], vd[1], vd[2]); 
+    auto vd = args.value("dims", std::vector<int>({132, 132, 132}));
+    r3::vector<int> dims(vd[0], vd[1], vd[2]); 
     double cutoff = args.value<double>("cutoff", 50);
 
-    matrix3d<double> M;
+    r3::matrix<double> M;
     M(0, 0) = M(1, 1) = M(2, 2) = 1.0;
     M(0, 1) = 0.1;
     M(0, 2) = 0.2;
     M(2, 0) = 0.3;
 
-    Gvec gvec(M, cutoff, Communicator::world(), false);
-    Gvec gvec_r(M, cutoff, Communicator::world(), true);
+    fft::Gvec gvec(M, cutoff, mpi::Communicator::world(), false);
+    fft::Gvec gvec_r(M, cutoff, mpi::Communicator::world(), true);
 
     if (gvec_r.num_gvec() * 2 != gvec.num_gvec() + 1) {
         return 1;
@@ -30,7 +29,7 @@ int run_test(cmd_args& args)
 int main(int argn, char** argv)
 {
     cmd_args args;
-    args.register_key("--dims=", "{vector3d<int>} FFT dimensions");
+    args.register_key("--dims=", "{vector<int>} FFT dimensions");
     args.register_key("--cutoff=", "{double} cutoff radius in G-space");
 
     args.parse_args(argn, argv);
