@@ -3,7 +3,7 @@
 #include <memory.hpp>
 #include <complex>
 #include <sys/time.h>
-#include "utils/timer.hpp"
+#include <random>
 
 using double_complex = std::complex<double>;
 using namespace sddk;
@@ -92,7 +92,6 @@ inline double wtime()
 
 double test_alloc(size_t n)
 {
-    utils::timer tt("test_alloc|std");
     double t0 = wtime();
     /* time to allocate + fill */
     char* ptr = (char*)std::malloc(n);
@@ -112,7 +111,6 @@ double test_alloc(size_t n)
 
 double test_alloc(size_t n, memory_pool& mp)
 {
-    utils::timer tt("test_alloc|mp");
     double t0 = wtime();
     /* time to allocate + fill */
     char* ptr = mp.allocate<char>(n);
@@ -169,7 +167,6 @@ void test6a()
     }
     std::cout << "std::malloc time: " << t0 << ", sddk::memory_pool time: " << t1 << "\n";
     mp.print();
-    utils::timer::print();
 }
 
 void test7()
@@ -183,7 +180,9 @@ void test7()
             auto n = (utils::rand() & 0b1111111111) + 1;
             v[i] = mp.allocate<double>(n);
         }
-        std::random_shuffle(v.begin(), v.end());
+        std::random_device rd;
+        std::mt19937 g(rd());
+        std::shuffle(v.begin(), v.end(), g);
         for (int i = 0; i < N; i++) {
             mp.free(v[i]);
         }
@@ -222,7 +221,6 @@ void test7()
 
 double test_alloc_array(size_t n)
 {
-    utils::timer tt("test_alloc|std");
     double t0 = wtime();
     /* time to allocate + fill */
     mdarray<char, 1> p(n);
@@ -242,10 +240,9 @@ double test_alloc_array(size_t n)
 
 double test_alloc_array(size_t n, memory_pool& mp)
 {
-    utils::timer tt("test_alloc|mp");
     double t0 = wtime();
     /* time to allocate + fill */
-    mdarray<char, 1> p(mp, n);
+    mdarray<char, 1> p(n, mp);
     p.zero();
     double t1 = wtime();
     /* time fo fill */
@@ -278,7 +275,6 @@ void test9()
         }
     }
     std::cout << "std::malloc time: " << t0 << ", sddk::memory_pool time: " << t1 << "\n";
-    utils::timer::print();
 }
 
 int run_test()
