@@ -9,7 +9,7 @@ using namespace sddk;
 template <typename T>
 int run_test_impl(cmd_args& args)
 {
-    matrix3d<double> lattice;
+    r3::matrix<double> lattice;
     lattice(0, 0) = 7;
     lattice(1, 1) = 7;
     lattice(2, 2) = 7;
@@ -43,11 +43,11 @@ int run_test_impl(cmd_args& args)
             auto ang = symmetry[isym].spg_op.euler_angles;
 
             /* random Cartesian vector */
-            vector3d<double> coord(double(rand()) / RAND_MAX, double(rand()) / RAND_MAX, double(rand()) / RAND_MAX);
-            auto scoord = SHT::spherical_coordinates(coord);
+            r3::vector<double> coord(double(rand()) / RAND_MAX, double(rand()) / RAND_MAX, double(rand()) / RAND_MAX);
+            auto scoord = r3::spherical_coordinates(coord);
             /* rotated coordinates */
             auto coord2 = dot(Rc, coord);
-            auto scoord2 = SHT::spherical_coordinates(coord2);
+            auto scoord2 = r3::spherical_coordinates(coord2);
 
             int lmax{10};
             sddk::mdarray<T, 1> ylm(utils::lmmax(lmax));
@@ -94,7 +94,7 @@ int run_test_impl(cmd_args& args)
 int run_test(cmd_args& args)
 {
     int result = run_test_impl<double>(args);
-    result += run_test_impl<double_complex>(args);
+    result += run_test_impl<std::complex<double>>(args);
     return result;
 }
 
