@@ -1,14 +1,14 @@
-#include "mpi/communicator.hpp"
-
 #include <mpi4py/mpi4py.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/pytypes.h>
 #include <pybind11/operators.h>
 #include <iostream>
 
+#include "mpi/communicator.hpp"
+
 namespace sirius {
 
-inline sddk::Communicator make_sirius_comm(pybind11::object py_comm)
+inline auto make_sirius_comm(pybind11::object py_comm)
 {
     PyObject* py_obj = py_comm.ptr();
     MPI_Comm* comm_p  = PyMPIComm_Get(py_obj);
@@ -18,14 +18,12 @@ inline sddk::Communicator make_sirius_comm(pybind11::object py_comm)
     int rank = -1;
     MPI_Comm_rank(*comm_p, &rank);
 
-    sddk::Communicator sirius_comm(*comm_p);
+    mpi::Communicator sirius_comm(*comm_p);
     return sirius_comm;
 }
 
-
-inline pybind11::handle make_pycomm(const sddk::Communicator& comm) {
-    return pybind11::handle(PyMPIComm_New(comm.mpi_comm()));
+inline pybind11::handle make_pycomm(const mpi::Communicator& comm) {
+    return pybind11::handle(PyMPIComm_New(comm.native()));
 }
 
-
 } // end namespace sirius
