@@ -40,6 +40,15 @@ enum class hdf5_access_t {
 template <typename T>
 struct hdf5_type_wrapper;
 
+template<>
+struct hdf5_type_wrapper<float>
+{
+    static hid_t type_id()
+    {
+        return H5T_NATIVE_FLOAT;
+    }
+};
+
 template<>
 struct hdf5_type_wrapper<double>
 {
@@ -49,12 +58,21 @@ struct hdf5_type_wrapper<double>
     }
 };
 
+template<>
+struct hdf5_type_wrapper<std::complex<float>>
+{
+    static hid_t type_id()
+    {
+        return H5T_NATIVE_FLOAT;
+    }
+};
+
 template<>
 struct hdf5_type_wrapper<std::complex<double>>
 {
     static hid_t type_id()
     {
-        return H5T_NATIVE_LDOUBLE;
+        return H5T_NATIVE_DOUBLE;
     }
 };
 
