@@ -23,7 +23,6 @@
  */
 #include "band.hpp"
 #include "davidson.hpp"
-#include "potential/potential.hpp"
 
 namespace sirius {
 
@@ -31,7 +30,7 @@ template <typename T>
 void
 Band::solve_full_potential(Hamiltonian_k<T>& Hk__, double itsol_tol__) const
 {
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(), FILE_LINE);
     if (ctx_.cfg().control().use_second_variation()) {
         /* solve non-magnetic Hamiltonian (so-called first variation) */
         auto& itso = ctx_.cfg().iterative_solver();
@@ -50,7 +49,7 @@ Band::solve_full_potential(Hamiltonian_k<T>& Hk__, double itsol_tol__) const
         throw std::runtime_error("not implemented");
         //diag_full_potential_single_variation();
     }
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(), FILE_LINE);
 }
 
 template
@@ -68,9 +67,9 @@ Band::solve_full_potential<float>(Hamiltonian_k<float>& Hk__, double itsol_tol__
 
 template <typename T, typename F>
 int
-Band::solve_pseudo_potential(Hamiltonian_k<real_type<T>>& Hk__, double itsol_tol__, double empy_tol__) const
+Band::solve_pseudo_potential(Hamiltonian_k<T>& Hk__, double itsol_tol__, double empy_tol__) const
 {
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(), FILE_LINE);
 
     int niter{0};
 
@@ -78,7 +77,7 @@ Band::solve_pseudo_potential(Hamiltonian_k<real_type<T>>& Hk__, double itsol_tol
     if (itso.type() == "exact") {
         if (ctx_.num_mag_dims() != 3) {
             for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
-                diag_pseudo_potential_exact<T>(ispn, Hk__);
+                diag_pseudo_potential_exact<T, F>(ispn, Hk__);
             }
         } else {
             STOP();
@@ -99,11 +98,13 @@ Band::solve_pseudo_potential(Hamiltonian_k<real_type<T>>& Hk__, double itsol_tol
             return tol;
         };
 
-        auto result = davidson<T, F, davidson_evp_t::hamiltonian>(Hk__, ctx_.num_bands(), ctx_.num_mag_dims(),
-                kp.spinor_wave_functions(), tolerance, itso.residual_tolerance(), itso.num_steps(),
-                itso.locking(), itso.subspace_size(), itso.converge_by_energy(), itso.extra_ortho(),
-                std::cout, 0);
+        std::stringstream s;
+        std::ostream* out = (kp.comm().rank() == 0) ? &std::cout : &s;
 
+        auto result = davidson<T, F, davidson_evp_t::hamiltonian>(Hk__, wf::num_bands(ctx_.num_bands()),
+                wf::num_mag_dims(ctx_.num_mag_dims()), kp.spinor_wave_functions(), tolerance,
+                itso.residual_tolerance(), itso.num_steps(), itso.locking(), itso.subspace_size(),
+                itso.converge_by_energy(), itso.extra_ortho(), *out, 0);
         niter = result.niter;
         for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
             for (int j = 0; j < ctx_.num_bands(); j++) {
@@ -114,13 +115,13 @@ Band::solve_pseudo_potential(Hamiltonian_k<real_type<T>>& Hk__, double itsol_tol
         RTE_THROW("unknown iterative solver type");
     }
 
-    /* check residuals */
-    if (ctx_.cfg().control().verification() >= 2) {
-        check_residuals<T>(Hk__);
-        check_wave_functions<T>(Hk__);
-    }
+    ///* check residuals */
+    //if (ctx_.cfg().control().verification() >= 2) {
+    //    check_residuals<T>(Hk__);
+    //    check_wave_functions<T>(Hk__);
+    //}
 
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(), FILE_LINE);
 
     return niter;
 }
@@ -131,14 +132,14 @@ Band::solve(K_point_set& kset__, Hamiltonian0<T>& H0__, double itsol_tol__) cons
 {
     PROFILE("sirius::Band::solve");
 
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(),FILE_LINE);
 
     double empy_tol{itsol_tol__};
     if (ctx_.cfg().iterative_solver().type() == "davidson") {
         empy_tol = std::max(itsol_tol__ * ctx_.cfg().settings().itsol_tol_ratio(),
                                    ctx_.cfg().iterative_solver().empty_states_tolerance());
-        ctx_.message(2, __function_name__, "iterative solver tolerance (occupied, empty) : %1.4e, %1.4e\n",
-                     itsol_tol__, itsol_tol__ + empy_tol);
+        ctx_.out(2, __func__) << "iterative solver tolerance (occupied, empty): " << itsol_tol__ << " "
+                              << itsol_tol__ + empy_tol << std::endl;
     }
 
     int num_dav_iter{0};
@@ -154,37 +155,40 @@ Band::solve(K_point_set& kset__, Hamiltonian0<T>& H0__, double itsol_tol__) cons
             if (ctx_.gamma_point() && (ctx_.so_correction() == false)) {
                 num_dav_iter += solve_pseudo_potential<T, F>(Hk, itsol_tol__, empy_tol);
             } else {
-                num_dav_iter += solve_pseudo_potential<std::complex<T>, std::complex<F>>(Hk, itsol_tol__, empy_tol);
+                num_dav_iter += solve_pseudo_potential<T, std::complex<F>>(Hk, itsol_tol__, empy_tol);
             }
         }
     }
     kset__.comm().allreduce(&num_dav_iter, 1);
     ctx_.num_itsol_steps(num_dav_iter);
     if (!ctx_.full_potential()) {
-        ctx_.message(2, __function_name__, "average number of iterations: %12.6f\n",
-                     static_cast<double>(num_dav_iter) / kset__.num_kpoints());
+        ctx_.out(2, __func__) << "average number of iterations: "
+                              << static_cast<double>(num_dav_iter) / kset__.num_kpoints() << std::endl;
     }
 
     /* synchronize eigen-values */
     kset__.sync_band<T, sync_band_t::energy>();
 
-    ctx_.message(2, __function_name__, "%s", "Lowest band energies\n");
-    if (ctx_.verbosity() >= 2 && ctx_.comm().rank() == 0) {
+    if (ctx_.verbosity() >= 2) {
+        std::stringstream s;
+        s << "Lowest band energies" << std::endl;
+        int nbnd = std::min(ctx_.cfg().control().num_bands_to_print(), ctx_.num_bands());
         for (int ik = 0; ik < kset__.num_kpoints(); ik++) {
-            std::printf("ik : %2i, ", ik);
-            for (int j = 0; j < std::min(ctx_.cfg().control().num_bands_to_print(), ctx_.num_bands()); j++) {
-                std::printf("%12.6f", kset__.get<T>(ik)->band_energy(j, 0));
+            s << "ik:" << std::setw(5) << ik;
+            for (int j = 0; j < nbnd; j++) {
+                s << utils::ffmt(12, 6) << kset__.get<T>(ik)->band_energy(j, 0);
             }
             if (ctx_.num_mag_dims() == 1) {
-                std::printf("\n         ");
-                for (int j = 0; j < std::min(ctx_.cfg().control().num_bands_to_print(), ctx_.num_bands()); j++) {
-                    std::printf("%12.6f", kset__.get<T>(ik)->band_energy(j, 1));
+                s << std::endl << "        ";
+                for (int j = 0; j < nbnd; j++) {
+                    s << utils::ffmt(12, 6) << kset__.get<T>(ik)->band_energy(j, 1);
                 }
             }
-            std::printf("\n");
+            s << std::endl;
         }
+        ctx_.message(2, __func__, s);
     }
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(), FILE_LINE);
 }
 
 template
