@@ -120,15 +120,15 @@ Simulation_context::init_fft_grid()
     /* create FFT driver for dense mesh (density and potential) */
     auto fft_grid = cfg().settings().fft_grid_size();
     if (fft_grid[0] * fft_grid[1] * fft_grid[2] == 0) {
-        fft_grid_ = sddk::get_min_fft_grid(pw_cutoff(), rlv);
+        fft_grid_ = fft::get_min_grid(pw_cutoff(), rlv);
         cfg().settings().fft_grid_size(fft_grid_);
     } else {
         /* else create a grid with user-specified dimensions */
-        fft_grid_ = sddk::FFT3D_grid(fft_grid);
+        fft_grid_ = fft::Grid(fft_grid);
     }
 
     /* create FFT grid for coarse mesh */
-    fft_coarse_grid_ = sddk::get_min_fft_grid(2 * gk_cutoff(), rlv);
+    fft_coarse_grid_ = fft::get_min_grid(2 * gk_cutoff(), rlv);
 }
 
 sddk::mdarray<double, 3>
@@ -148,23 +148,23 @@ Simulation_context::generate_sbessel_mt(int lmax__) const
     return sbessel_mt;
 }
 
-sddk::matrix<double_complex>
+sddk::matrix<std::complex<double>>
 Simulation_context::generate_gvec_ylm(int lmax__)
 {
     PROFILE("sirius::Simulation_context::generate_gvec_ylm");
 
-    sddk::matrix<double_complex> gvec_ylm(utils::lmmax(lmax__), gvec().count(), sddk::memory_t::host, "gvec_ylm");
+    sddk::matrix<std::complex<double>> gvec_ylm(utils::lmmax(lmax__), gvec().count(), sddk::memory_t::host, "gvec_ylm");
     #pragma omp parallel for schedule(static)
     for (int igloc = 0; igloc < gvec().count(); igloc++) {
-        auto rtp = SHT::spherical_coordinates(gvec().gvec_cart<sddk::index_domain_t::local>(igloc));
+        auto rtp = r3::spherical_coordinates(gvec().gvec_cart<sddk::index_domain_t::local>(igloc));
         sf::spherical_harmonics(lmax__, rtp[1], rtp[2], &gvec_ylm(0, igloc));
     }
     return gvec_ylm;
 }
 
-sddk::mdarray<double_complex, 2>
-Simulation_context::sum_fg_fl_yg(int lmax__, double_complex const* fpw__, sddk::mdarray<double, 3>& fl__,
-                                 sddk::matrix<double_complex>& gvec_ylm__)
+sddk::mdarray<std::complex<double>, 2>
+Simulation_context::sum_fg_fl_yg(int lmax__, std::complex<double> const* fpw__, sddk::mdarray<double, 3>& fl__,
+                                 sddk::matrix<std::complex<double>>& gvec_ylm__)
 {
     PROFILE("sirius::Simulation_context::sum_fg_fl_yg");
 
@@ -177,36 +177,36 @@ Simulation_context::sum_fg_fl_yg(int lmax__, double_complex const* fpw__, sddk::
 
     int lmmax = utils::lmmax(lmax__);
     /* resuling matrix */
-    sddk::mdarray<double_complex, 2> flm(lmmax, unit_cell().num_atoms());
+    sddk::mdarray<std::complex<double>, 2> flm(lmmax, unit_cell().num_atoms());
 
-    sddk::matrix<double_complex> phase_factors;
-    sddk::matrix<double_complex> zm;
-    sddk::matrix<double_complex> tmp;
+    sddk::matrix<std::complex<double>> phase_factors;
+    sddk::matrix<std::complex<double>> zm;
+    sddk::matrix<std::complex<double>> tmp;
 
     switch (processing_unit()) {
         case sddk::device_t::CPU: {
             auto& mp      = get_memory_pool(sddk::memory_t::host);
-            phase_factors = sddk::matrix<double_complex>(ngv_loc, na_max, mp);
-            zm            = sddk::matrix<double_complex>(lmmax, ngv_loc, mp);
-            tmp           = sddk::matrix<double_complex>(lmmax, na_max, mp);
+            phase_factors = sddk::matrix<std::complex<double>>(ngv_loc, na_max, mp);
+            zm            = sddk::matrix<std::complex<double>>(lmmax, ngv_loc, mp);
+            tmp           = sddk::matrix<std::complex<double>>(lmmax, na_max, mp);
             break;
         }
         case sddk::device_t::GPU: {
             auto& mp      = get_memory_pool(sddk::memory_t::host);
             auto& mpd     = get_memory_pool(sddk::memory_t::device);
-            phase_factors = sddk::matrix<double_complex>(nullptr, ngv_loc, na_max);
+            phase_factors = sddk::matrix<std::complex<double>>(nullptr, ngv_loc, na_max);
             phase_factors.allocate(mpd);
-            zm = sddk::matrix<double_complex>(lmmax, ngv_loc, mp);
+            zm = sddk::matrix<std::complex<double>>(lmmax, ngv_loc, mp);
             zm.allocate(mpd);
-            tmp = sddk::matrix<double_complex>(lmmax, na_max, mp);
+            tmp = sddk::matrix<std::complex<double>>(lmmax, na_max, mp);
             tmp.allocate(mpd);
             break;
         }
     }
 
-    std::vector<double_complex> zil(lmax__ + 1);
+    std::vector<std::complex<double>> zil(lmax__ + 1);
     for (int l = 0; l <= lmax__; l++) {
-        zil[l] = std::pow(double_complex(0, 1), l);
+        zil[l] = std::pow(std::complex<double>(0, 1), l);
     }
 
     for (int iat = 0; iat < unit_cell().num_atom_types(); iat++) {
@@ -216,7 +216,7 @@ Simulation_context::sum_fg_fl_yg(int lmax__, double_complex const* fpw__, sddk::
         #pragma omp parallel for schedule(static)
         for (int igloc = 0; igloc < ngv_loc; igloc++) {
             for (int l = 0, lm = 0; l <= lmax__; l++) {
-                double_complex z = fourpi * fl__(l, igloc, iat) * zil[l] * fpw__[igloc];
+                std::complex<double> z = fourpi * fl__(l, igloc, iat) * zil[l] * fpw__[igloc];
                 for (int m = -l; m <= l; m++, lm++) {
                     zm(lm, igloc) = z * std::conj(gvec_ylm__(lm, igloc));
                 }
@@ -226,18 +226,18 @@ Simulation_context::sum_fg_fl_yg(int lmax__, double_complex const* fpw__, sddk::
         PROFILE_START("sirius::Simulation_context::sum_fg_fl_yg|mul");
         switch (processing_unit()) {
             case sddk::device_t::CPU: {
-                sddk::linalg(sddk::linalg_t::blas)
-                    .gemm('N', 'N', lmmax, na, ngv_loc, &sddk::linalg_const<double_complex>::one(), zm.at(sddk::memory_t::host),
+                la::wrap(la::lib_t::blas)
+                    .gemm('N', 'N', lmmax, na, ngv_loc, &la::constant<std::complex<double>>::one(), zm.at(sddk::memory_t::host),
                           zm.ld(), phase_factors.at(sddk::memory_t::host), phase_factors.ld(),
-                          &sddk::linalg_const<double_complex>::zero(), tmp.at(sddk::memory_t::host), tmp.ld());
+                          &la::constant<std::complex<double>>::zero(), tmp.at(sddk::memory_t::host), tmp.ld());
                 break;
             }
             case sddk::device_t::GPU: {
                 zm.copy_to(sddk::memory_t::device);
-                sddk::linalg(sddk::linalg_t::gpublas)
-                    .gemm('N', 'N', lmmax, na, ngv_loc, &sddk::linalg_const<double_complex>::one(), zm.at(sddk::memory_t::device),
+                la::wrap(la::lib_t::gpublas)
+                    .gemm('N', 'N', lmmax, na, ngv_loc, &la::constant<std::complex<double>>::one(), zm.at(sddk::memory_t::device),
                           zm.ld(), phase_factors.at(sddk::memory_t::device), phase_factors.ld(),
-                          &sddk::linalg_const<double_complex>::zero(), tmp.at(sddk::memory_t::device), tmp.ld());
+                          &la::constant<std::complex<double>>::zero(), tmp.at(sddk::memory_t::device), tmp.ld());
                 tmp.copy_to(sddk::memory_t::host);
                 break;
             }
@@ -279,28 +279,6 @@ Simulation_context::ewald_lambda() const
     return lambda;
 }
 
-sddk::splindex<sddk::splindex_t::block>
-Simulation_context::split_gvec_local() const
-{
-    /* local number of G-vectors for this MPI rank */
-    int ngv_loc = gvec().count();
-    /* estimate number of G-vectors in a block */
-    int ld{-1};
-    for (int iat = 0; iat < unit_cell().num_atom_types(); iat++) {
-        int nat = unit_cell().atom_type(iat).num_atoms();
-        int nbf = unit_cell().atom_type(iat).mt_basis_size();
-
-        ld = std::max(ld, std::max(nbf * (nbf + 1) / 2, nat));
-    }
-    /* limit the size of relevant array to ~1Gb */
-    int ngv_b = (1 << 30) / sizeof(double_complex) / ld;
-    ngv_b     = std::max(1, std::min(ngv_loc, ngv_b));
-    /* number of blocks of G-vectors */
-    int nb = ngv_loc / ngv_b;
-    /* split local number of G-vectors between blocks */
-    return sddk::splindex<sddk::splindex_t::block>(ngv_loc, nb, 0);
-}
-
 void
 Simulation_context::initialize()
 {
@@ -361,22 +339,6 @@ Simulation_context::initialize()
     /* initialize MPI communicators */
     init_comm();
 
-    auto print_mpi_layout = env::print_mpi_layout();
-
-    if (verbosity() >= 3 || print_mpi_layout) {
-        sddk::pstdout pout(comm());
-        if (comm().rank() == 0) {
-            pout << "MPI rank placement" << std::endl;
-            pout << "------------------" << std::endl;
-        }
-        pout << "rank: " << comm().rank()
-             << ", comm_band_rank: " << comm_band().rank()
-             << ", comm_k_rank: " << comm_k().rank()
-             << ", hostname: " << utils::hostname()
-             << ", mpi processor name: " << sddk::Communicator::processor_name() << std::endl;
-        rte::ostream(this->out(), "info") << pout.flush(0);
-    }
-
     switch (processing_unit()) {
         case sddk::device_t::CPU: {
             host_memory_t_ = sddk::memory_t::host;
@@ -421,7 +383,7 @@ Simulation_context::initialize()
 
     /* check the lattice symmetries */
     if (use_symmetry()) {
-        auto lv = matrix3d<double>(unit_cell().lattice_vectors());
+        auto lv = r3::matrix<double>(unit_cell().lattice_vectors());
 
         auto lat_sym = find_lat_sym(lv, cfg().control().spglib_tolerance());
 
@@ -573,6 +535,7 @@ Simulation_context::initialize()
         }
     }
 
+    /* environment variable has a highest priority */
     auto ev_str = env::get_ev_solver();
     if (ev_str.size()) {
         evsn[0] = ev_str;
@@ -582,8 +545,8 @@ Simulation_context::initialize()
     std_evp_solver_name(evsn[0]);
     gen_evp_solver_name(evsn[1]);
 
-    std_evp_solver_ = Eigensolver_factory(std_evp_solver_name());
-    gen_evp_solver_ = Eigensolver_factory(gen_evp_solver_name());
+    std_evp_solver_ = la::Eigensolver_factory(std_evp_solver_name());
+    gen_evp_solver_ = la::Eigensolver_factory(gen_evp_solver_name());
 
     auto& std_solver = std_evp_solver();
     auto& gen_solver = gen_evp_solver();
@@ -594,9 +557,9 @@ Simulation_context::initialize()
 
     /* setup BLACS grid */
     if (std_solver.is_parallel()) {
-        blacs_grid_ = std::make_unique<sddk::BLACS_grid>(comm_band(), npr, npc);
+        blacs_grid_ = std::make_unique<la::BLACS_grid>(comm_band(), npr, npc);
     } else {
-        blacs_grid_ = std::make_unique<sddk::BLACS_grid>(sddk::Communicator::self(), 1, 1);
+        blacs_grid_ = std::make_unique<la::BLACS_grid>(mpi::Communicator::self(), 1, 1);
     }
 
     /* setup the cyclic block size */
@@ -646,6 +609,30 @@ Simulation_context::initialize()
         this->cfg().control().print_checksum(true);
     }
 
+    auto print_mpi_layout = env::print_mpi_layout();
+
+    if (verbosity() >= 3 || print_mpi_layout) {
+        mpi::pstdout pout(comm());
+        if (comm().rank() == 0) {
+            pout << "MPI rank placement" << std::endl;
+            pout << utils::hbar(136, '-') << std::endl;
+            pout << "             |  comm tot, band, k | comm fft, ortho | mpi_grid tot, row, col | blacs tot, row, col" << std::endl;
+        }
+        pout << std::setw(12) << utils::hostname() << " | "
+             << std::setw(6) << comm().rank()
+             << std::setw(6) << comm_band().rank()
+             << std::setw(6) << comm_k().rank() << " | "
+             << std::setw(6) << comm_fft_coarse().rank()
+             << std::setw(6) << comm_band_ortho_fft_coarse().rank() << "    |   "
+             << std::setw(6) << mpi_grid_->communicator(3).rank()
+             << std::setw(6) << mpi_grid_->communicator(1 << 0).rank()
+             << std::setw(6) << mpi_grid_->communicator(1 << 1).rank() << "   | "
+             << std::setw(6) << blacs_grid().comm().rank()
+             << std::setw(6) << blacs_grid().comm_row().rank()
+             << std::setw(6) << blacs_grid().comm_col().rank() << std::endl;
+        rte::ostream(this->out(), "info") << pout.flush(0);
+    }
+
     initialized_ = true;
     cfg().lock();
 }
@@ -669,13 +656,13 @@ Simulation_context::print_info(std::ostream& out__) const
            << "number of MPI ranks           : " << this->comm().size() << std::endl;
         if (mpi_grid_) {
             os << "MPI grid                      :";
-            for (int i = 0; i < mpi_grid_->num_dimensions(); i++) {
+            for (int i : {0, 1}) {
                 os << " " << mpi_grid_->communicator(1 << i).size();
             }
             os << std::endl;
         }
         os << "maximum number of OMP threads : " << omp_get_max_threads() << std::endl
-           << "number of MPI ranks per node  : " << sddk::num_ranks_per_node() << std::endl
+           << "number of MPI ranks per node  : " << mpi::num_ranks_per_node() << std::endl
            << "page size (Kb)                : " << (utils::get_page_size() >> 10) << std::endl
            << "number of pages               : " << utils::get_num_pages() << std::endl
            << "available memory (GB)         : " << (utils::get_total_memory() >> 30) << std::endl;
@@ -685,9 +672,9 @@ Simulation_context::print_info(std::ostream& out__) const
         rte::ostream os(out__, "fft");
         std::string headers[]       = {"FFT context for density and potential", "FFT context for coarse grid"};
         double cutoffs[]            = {pw_cutoff(), 2 * gk_cutoff()};
-        sddk::Communicator const* comms[] = {&comm_fft(), &comm_fft_coarse()};
-        sddk::FFT3D_grid fft_grids[]      = {this->fft_grid_, this->fft_coarse_grid_};
-        sddk::Gvec const* gvecs[]         = {&gvec(), &gvec_coarse()};
+        mpi::Communicator const* comms[] = {&comm_fft(), &comm_fft_coarse()};
+        fft::Grid fft_grids[]      = {this->fft_grid_, this->fft_coarse_grid_};
+        fft::Gvec const* gvecs[]         = {&gvec(), &gvec_coarse()};
 
         for (int i = 0; i < 2; i++) {
             os << headers[i] << std::endl
@@ -709,7 +696,6 @@ Simulation_context::print_info(std::ostream& out__) const
                << "  number of G-shells                    : " << gvecs[i]->num_shells() << std::endl
                << std::endl;
         }
-        os << "number of local G-vector blocks: " << split_gvec_local().num_ranks() << std::endl;
         os << std::endl;
     }
     {
@@ -771,14 +757,14 @@ Simulation_context::print_info(std::ostream& out__) const
         }
 
         std::string evsn[] = {"standard eigen-value solver        : ", "generalized eigen-value solver     : "};
-        ev_solver_t evst[] = {std_evp_solver().type(), gen_evp_solver().type()};
-        std::map<ev_solver_t, std::string> const evsm = {
-            {ev_solver_t::lapack, "LAPACK"},
-            {ev_solver_t::scalapack, "ScaLAPACK"},
-            {ev_solver_t::elpa, "ELPA"},
-            {ev_solver_t::magma, "MAGMA"},
-            {ev_solver_t::magma_gpu, "MAGMA with GPU pointers"},
-            {ev_solver_t::cusolver, "cuSOLVER"}
+        la::ev_solver_t evst[] = {std_evp_solver().type(), gen_evp_solver().type()};
+        std::map<la::ev_solver_t, std::string> const evsm = {
+            {la::ev_solver_t::lapack, "LAPACK"},
+            {la::ev_solver_t::scalapack, "ScaLAPACK"},
+            {la::ev_solver_t::elpa, "ELPA"},
+            {la::ev_solver_t::magma, "MAGMA"},
+            {la::ev_solver_t::magma_gpu, "MAGMA with GPU pointers"},
+            {la::ev_solver_t::cusolver, "cuSOLVER"}
         };
         for (int i = 0; i < 2; i++) {
             os << evsn[i] << evsm.at(evst[i]) << std::endl;
@@ -895,7 +881,7 @@ Simulation_context::print_info(std::ostream& out__) const
         */
         size_t tot_size = (num_bands() * num_spins() + 2 * num_bands() * num_sc + 3 * num_phi * num_sc +
                            num_bands() * num_sc + num_bands()) *
-                          ngk * sizeof(double_complex);
+                          ngk * sizeof(std::complex<double>);
         os << "approximate memory consumption of Davidson solver: "
            << static_cast<int>((tot_size / comm_band().size()) >> 20) << " Mb/rank" << std::endl;
 
@@ -909,20 +895,20 @@ Simulation_context::print_info(std::ostream& out__) const
                will be stored on GPU and computation will be overlapped with transfer of the  next augmentation
                operator */
             // TODO: optimize generated_rho_aug() for less memory consumption
-            size_t size_aug = nb * ngloc * sizeof(double_complex);
+            size_t size_aug = nb * ngloc * sizeof(std::complex<double>);
             if (unit_cell().num_atom_types() > 1) {
                 size_aug *= 2;
             }
 
             /* and two more arrays will be allocated in generate_rho_aug() with 1Gb maximum size each */
-            size_t size1 = nb * ngloc * sizeof(double_complex);
+            size_t size1 = nb * ngloc * sizeof(std::complex<double>);
             size1        = std::min(size1, static_cast<size_t>(1 << 30));
 
             int max_atoms{0};
             for (int iat = 0; iat < unit_cell().num_atom_types(); iat++) {
                 max_atoms = std::max(max_atoms, unit_cell().atom_type(iat).num_atoms());
             }
-            size_t size2 = max_atoms * ngloc * sizeof(double_complex);
+            size_t size2 = max_atoms * ngloc * sizeof(std::complex<double>);
             size2        = std::min(size2, static_cast<size_t>(1 << 30));
 
             size_aug += (size1 + size2);
@@ -962,21 +948,22 @@ Simulation_context::update()
        the next time only reciprocal lattice of the G-vectors is updated */
     if (!gvec_coarse_) {
         /* create list of coarse G-vectors */
-        gvec_coarse_ = std::make_unique<sddk::Gvec>(rlv, 2 * gk_cutoff(), comm(), cfg().control().reduce_gvec());
+        gvec_coarse_ = std::make_unique<fft::Gvec>(rlv, 2 * gk_cutoff(), comm(), cfg().control().reduce_gvec(),
+                cfg().control().spglib_tolerance());
         /* create FFT friendly partiton */
-        gvec_coarse_fft_ = std::make_shared<sddk::Gvec_fft>(*gvec_coarse_, comm_fft_coarse(),
+        gvec_coarse_fft_ = std::make_shared<fft::Gvec_fft>(*gvec_coarse_, comm_fft_coarse(),
                 comm_ortho_fft_coarse());
 
-        auto spl_z = split_fft_z(fft_coarse_grid_[2], comm_fft_coarse());
+        auto spl_z = fft::split_z_dimension(fft_coarse_grid_[2], comm_fft_coarse());
 
         /* create spfft buffer for coarse transform */
         spfft_grid_coarse_ = std::make_unique<spfft::Grid>(fft_coarse_grid_[0], fft_coarse_grid_[1],
-                fft_coarse_grid_[2], gvec_coarse_fft_->zcol_count_fft(),
-                spl_z.local_size(), spfft_pu, -1, comm_fft_coarse().mpi_comm(), SPFFT_EXCH_DEFAULT);
+                fft_coarse_grid_[2], gvec_coarse_fft_->zcol_count(),
+                spl_z.local_size(), spfft_pu, -1, comm_fft_coarse().native(), SPFFT_EXCH_DEFAULT);
 #ifdef USE_FP32
         spfft_grid_coarse_float_ = std::make_unique<spfft::GridFloat>(fft_coarse_grid_[0], fft_coarse_grid_[1],
-                fft_coarse_grid_[2], gvec_coarse_fft_->zcol_count_fft(), spl_z.local_size(), spfft_pu, -1,
-                comm_fft_coarse().mpi_comm(), SPFFT_EXCH_DEFAULT);
+                fft_coarse_grid_[2], gvec_coarse_fft_->zcol_count(), spl_z.local_size(), spfft_pu, -1,
+                comm_fft_coarse().native(), SPFFT_EXCH_DEFAULT);
 #endif
         /* create spfft transformations */
         const auto fft_type_coarse = gvec_coarse().reduced() ? SPFFT_TRANS_R2C : SPFFT_TRANS_C2C;
@@ -986,12 +973,12 @@ Simulation_context::update()
         /* create actual transform object */
         spfft_transform_coarse_.reset(new spfft::Transform(spfft_grid_coarse_->create_transform(
             spfft_pu, fft_type_coarse, fft_coarse_grid_[0], fft_coarse_grid_[1], fft_coarse_grid_[2],
-            spl_z.local_size(), gvec_coarse_fft_->gvec_count_fft(), SPFFT_INDEX_TRIPLETS,
+            spl_z.local_size(), gvec_coarse_fft_->count(), SPFFT_INDEX_TRIPLETS,
             gv.at(sddk::memory_t::host))));
 #ifdef USE_FP32
         spfft_transform_coarse_float_.reset(new spfft::TransformFloat(spfft_grid_coarse_float_->create_transform(
             spfft_pu, fft_type_coarse, fft_coarse_grid_[0], fft_coarse_grid_[1], fft_coarse_grid_[2],
-            spl_z.local_size(), gvec_coarse_fft_->gvec_count_fft(), SPFFT_INDEX_TRIPLETS,
+            spl_z.local_size(), gvec_coarse_fft_->count(), SPFFT_INDEX_TRIPLETS,
             gv.at(sddk::memory_t::host))));
 #endif
     } else {
@@ -1000,20 +987,20 @@ Simulation_context::update()
 
     /* create a list of G-vectors for dense FFT grid; G-vectors are divided between all available MPI ranks.*/
     if (!gvec_) {
-        gvec_     = std::make_shared<sddk::Gvec>(pw_cutoff(), *gvec_coarse_);
-        gvec_fft_ = std::make_shared<sddk::Gvec_fft>(*gvec_, comm_fft(), comm_ortho_fft());
+        gvec_     = std::make_shared<fft::Gvec>(pw_cutoff(), *gvec_coarse_);
+        gvec_fft_ = std::make_shared<fft::Gvec_fft>(*gvec_, comm_fft(), comm_ortho_fft());
 
-        auto spl_z = split_fft_z(fft_grid_[2], comm_fft());
+        auto spl_z = fft::split_z_dimension(fft_grid_[2], comm_fft());
 
         /* create spfft buffer for fine-grained transform */
         spfft_grid_ = std::unique_ptr<spfft::Grid>(
             new spfft::Grid(fft_grid_[0], fft_grid_[1], fft_grid_[2],
-                            gvec_fft_->zcol_count_fft(), spl_z.local_size(), spfft_pu, -1,
-                            comm_fft().mpi_comm(), SPFFT_EXCH_DEFAULT));
+                            gvec_fft_->zcol_count(), spl_z.local_size(), spfft_pu, -1,
+                            comm_fft().native(), SPFFT_EXCH_DEFAULT));
 #if defined(USE_FP32)
         spfft_grid_float_ = std::unique_ptr<spfft::GridFloat>(
-            new spfft::GridFloat(fft_grid_[0], fft_grid_[1], fft_grid_[2], gvec_fft_->zcol_count_fft(),
-                                 spl_z.local_size(), spfft_pu, -1, comm_fft().mpi_comm(), SPFFT_EXCH_DEFAULT));
+            new spfft::GridFloat(fft_grid_[0], fft_grid_[1], fft_grid_[2], gvec_fft_->zcol_count(),
+                                 spl_z.local_size(), spfft_pu, -1, comm_fft().native(), SPFFT_EXCH_DEFAULT));
 #endif
         const auto fft_type = gvec().reduced() ? SPFFT_TRANS_R2C : SPFFT_TRANS_C2C;
 
@@ -1021,11 +1008,11 @@ Simulation_context::update()
 
         spfft_transform_.reset(new spfft::Transform(spfft_grid_->create_transform(
             spfft_pu, fft_type, fft_grid_[0], fft_grid_[1], fft_grid_[2],
-            spl_z.local_size(), gvec_fft_->gvec_count_fft(), SPFFT_INDEX_TRIPLETS, gv.at(sddk::memory_t::host))));
+            spl_z.local_size(), gvec_fft_->count(), SPFFT_INDEX_TRIPLETS, gv.at(sddk::memory_t::host))));
 #if defined(USE_FP32)
         spfft_transform_float_.reset(new spfft::TransformFloat(spfft_grid_float_->create_transform(
             spfft_pu, fft_type, fft_grid_[0], fft_grid_[1], fft_grid_[2], spl_z.local_size(),
-            gvec_fft_->gvec_count_fft(), SPFFT_INDEX_TRIPLETS, gv.at(sddk::memory_t::host))));
+            gvec_fft_->count(), SPFFT_INDEX_TRIPLETS, gv.at(sddk::memory_t::host))));
 #endif
 
         /* copy G-vectors to GPU; this is done once because Miller indices of G-vectors
@@ -1054,7 +1041,7 @@ Simulation_context::update()
     /* After each update of the lattice vectors we might get a different set of G-vector shells.
      * Always update the mapping between the canonical FFT distribution and "local G-shells"
      * distribution which is used in symmetriezation of lattice periodic functions. */
-    remap_gvec_ = std::make_unique<sddk::Gvec_shells>(gvec());
+    remap_gvec_ = std::make_unique<fft::Gvec_shells>(gvec());
 
     /* check symmetry of G-vectors */
     if (unit_cell().num_atoms() != 0 && use_symmetry() && cfg().control().verification() >= 1) {
@@ -1101,25 +1088,25 @@ Simulation_context::update()
     }
 
     /* recompute phase factors for atoms */
-    phase_factors_ = sddk::mdarray<double_complex, 3>(3, limits, unit_cell().num_atoms(), sddk::memory_t::host, "phase_factors_");
+    phase_factors_ = sddk::mdarray<std::complex<double>, 3>(3, limits, unit_cell().num_atoms(), sddk::memory_t::host, "phase_factors_");
     #pragma omp parallel for
     for (int i = limits.first; i <= limits.second; i++) {
         for (int ia = 0; ia < unit_cell().num_atoms(); ia++) {
             auto pos = unit_cell().atom(ia).position();
             for (int x : {0, 1, 2}) {
-                phase_factors_(x, i, ia) = std::exp(double_complex(0.0, twopi * (i * pos[x])));
+                phase_factors_(x, i, ia) = std::exp(std::complex<double>(0.0, twopi * (i * pos[x])));
             }
         }
     }
 
     /* recompute phase factors for atom types */
-    phase_factors_t_ = sddk::mdarray<double_complex, 2>(gvec().count(), unit_cell().num_atom_types());
+    phase_factors_t_ = sddk::mdarray<std::complex<double>, 2>(gvec().count(), unit_cell().num_atom_types());
     #pragma omp parallel for schedule(static)
     for (int igloc = 0; igloc < gvec().count(); igloc++) {
         /* global index of G-vector */
         int ig = gvec().offset() + igloc;
         for (int iat = 0; iat < unit_cell().num_atom_types(); iat++) {
-            double_complex z(0, 0);
+            std::complex<double> z(0, 0);
             for (int ia = 0; ia < unit_cell().atom_type(iat).num_atoms(); ia++) {
                 z += gvec_phase_factor(ig, unit_cell().atom_type(iat).atom_id(ia));
             }
@@ -1128,34 +1115,25 @@ Simulation_context::update()
     }
 
     if (use_symmetry()) {
-        sym_phase_factors_ = sddk::mdarray<double_complex, 3>(3, limits, unit_cell().symmetry().size());
+        sym_phase_factors_ = sddk::mdarray<std::complex<double>, 3>(3, limits, unit_cell().symmetry().size());
 
         #pragma omp parallel for
         for (int i = limits.first; i <= limits.second; i++) {
             for (int isym = 0; isym < unit_cell().symmetry().size(); isym++) {
                 auto t = unit_cell().symmetry()[isym].spg_op.t;
                 for (int x : {0, 1, 2}) {
-                    sym_phase_factors_(x, i, isym) = std::exp(double_complex(0.0, twopi * (i * t[x])));
+                    sym_phase_factors_(x, i, isym) = std::exp(std::complex<double>(0.0, twopi * (i * t[x])));
                 }
             }
         }
     }
 
-    /* precompute some G-vector related arrays */
-    gvec_tp_ = sddk::mdarray<double, 2>(gvec().count(), 2, sddk::memory_t::host, "gvec_tp_");
-    #pragma omp parallel for schedule(static)
-    for (int igloc = 0; igloc < gvec().count(); igloc++) {
-        auto rtp           = SHT::spherical_coordinates(gvec().gvec_cart<sddk::index_domain_t::local>(igloc));
-        gvec_tp_(igloc, 0) = rtp[1];
-        gvec_tp_(igloc, 1) = rtp[2];
-    }
-
     switch (this->processing_unit()) {
         case sddk::device_t::CPU: {
             break;
         }
         case sddk::device_t::GPU: {
-            gvec_tp_.allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
+            gvec_->gvec_tp().allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
             break;
         }
     }
@@ -1167,7 +1145,7 @@ Simulation_context::update()
         for (int igloc = 0; igloc < gvec().count(); igloc++) {
             new_pw_cutoff = std::max(new_pw_cutoff, gvec().gvec_len<sddk::index_domain_t::local>(igloc));
         }
-        gvec().comm().allreduce<double, sddk::mpi_op_t::max>(&new_pw_cutoff, 1);
+        gvec().comm().allreduce<double, mpi::op_t::max>(&new_pw_cutoff, 1);
         /* estimate new G+k-vectors cutoff */
         double new_gk_cutoff = this->gk_cutoff();
         if (new_pw_cutoff > this->pw_cutoff()) {
@@ -1241,24 +1219,10 @@ Simulation_context::update()
                 new Radial_integrals_atomic_wf<true>(unit_cell(), new_gk_cutoff, 20, idxr_wf, ps_wf, ps_atomic_wf_ri_djl_callback_));
         }
 
-        /* update augmentation operator */
-        sddk::memory_pool* mp{nullptr};
-        sddk::memory_pool* mpd{nullptr};
-        switch (this->processing_unit()) {
-            case sddk::device_t::CPU: {
-                mp = &get_memory_pool(sddk::memory_t::host);
-                break;
-            }
-            case sddk::device_t::GPU: {
-                mp  = &get_memory_pool(sddk::memory_t::host_pinned);
-                mpd = &get_memory_pool(sddk::memory_t::device);
-                break;
-            }
-        }
         for (int iat = 0; iat < unit_cell().num_atom_types(); iat++) {
             if (unit_cell().atom_type(iat).augment() && unit_cell().atom_type(iat).num_atoms() > 0) {
-                augmentation_op_[iat] = std::make_unique<Augmentation_operator>(unit_cell().atom_type(iat), gvec());
-                augmentation_op_[iat]->generate_pw_coeffs(aug_ri(), gvec_tp_, *mp, mpd);
+                augmentation_op_[iat] = std::make_unique<Augmentation_operator>(unit_cell().atom_type(iat), gvec(), aug_ri(), aug_ri_djl());
+                augmentation_op_[iat]->generate_pw_coeffs();
             } else {
                 augmentation_op_[iat] = nullptr;
             }
@@ -1329,7 +1293,7 @@ Simulation_context::create_storage_file() const
 }
 
 void
-Simulation_context::generate_phase_factors(int iat__, sddk::mdarray<double_complex, 2>& phase_factors__) const
+Simulation_context::generate_phase_factors(int iat__, sddk::mdarray<std::complex<double>, 2>& phase_factors__) const
 {
     PROFILE("sirius::Simulation_context::generate_phase_factors");
     int na = unit_cell().atom_type(iat__).num_atoms();
@@ -1372,27 +1336,27 @@ Simulation_context::init_atoms_to_grid_idx(double R__)
 
     atoms_to_grid_idx_.resize(unit_cell().num_atoms());
 
-    vector3d<double> delta(1.0 / spfft<double>().dim_x(), 1.0 / spfft<double>().dim_y(), 1.0 / spfft<double>().dim_z());
+    r3::vector<double> delta(1.0 / spfft<double>().dim_x(), 1.0 / spfft<double>().dim_y(), 1.0 / spfft<double>().dim_z());
 
     int z_off = spfft<double>().local_z_offset();
-    vector3d<int> grid_beg(0, 0, z_off);
-    vector3d<int> grid_end(spfft<double>().dim_x(), spfft<double>().dim_y(), z_off + spfft<double>().local_z_length());
-    std::vector<vector3d<double>> verts_cart{{-R, -R, -R}, {R, -R, -R}, {-R, R, -R}, {R, R, -R},
+    r3::vector<int> grid_beg(0, 0, z_off);
+    r3::vector<int> grid_end(spfft<double>().dim_x(), spfft<double>().dim_y(), z_off + spfft<double>().local_z_length());
+    std::vector<r3::vector<double>> verts_cart{{-R, -R, -R}, {R, -R, -R}, {-R, R, -R}, {R, R, -R},
                                              {-R, -R, R},  {R, -R, R},  {-R, R, R},  {R, R, R}};
 
-    auto bounds_box = [&](vector3d<double> pos) {
-        std::vector<vector3d<double>> verts;
+    auto bounds_box = [&](r3::vector<double> pos) {
+        std::vector<r3::vector<double>> verts;
 
         /* pos is a position of atom */
         for (auto v : verts_cart) {
             verts.push_back(pos + unit_cell().get_fractional_coordinates(v));
         }
 
-        std::pair<vector3d<int>, vector3d<int>> bounds_ind;
+        std::pair<r3::vector<int>, r3::vector<int>> bounds_ind;
 
         for (int x : {0, 1, 2}) {
             std::sort(verts.begin(), verts.end(),
-                      [x](vector3d<double>& a, vector3d<double>& b) { return a[x] < b[x]; });
+                      [x](r3::vector<double>& a, r3::vector<double>& b) { return a[x] < b[x]; });
             bounds_ind.first[x]  = std::max(static_cast<int>(verts[0][x] / delta[x]) - 1, grid_beg[x]);
             bounds_ind.second[x] = std::min(static_cast<int>(verts[5][x] / delta[x]) + 1, grid_end[x]);
         }
@@ -1408,7 +1372,7 @@ Simulation_context::init_atoms_to_grid_idx(double R__)
         for (int t0 = -1; t0 <= 1; t0++) {
             for (int t1 = -1; t1 <= 1; t1++) {
                 for (int t2 = -1; t2 <= 1; t2++) {
-                    auto pos = unit_cell().atom(ia).position() + vector3d<double>(t0, t1, t2);
+                    auto pos = unit_cell().atom(ia).position() + r3::vector<double>(t0, t1, t2);
 
                     /* find the small box around this atom */
                     auto box = bounds_box(pos);
@@ -1416,7 +1380,7 @@ Simulation_context::init_atoms_to_grid_idx(double R__)
                     for (int j0 = box.first[0]; j0 < box.second[0]; j0++) {
                         for (int j1 = box.first[1]; j1 < box.second[1]; j1++) {
                             for (int j2 = box.first[2]; j2 < box.second[2]; j2++) {
-                                auto v = pos - vector3d<double>(delta[0] * j0, delta[1] * j1, delta[2] * j2);
+                                auto v = pos - r3::vector<double>(delta[0] * j0, delta[1] * j1, delta[2] * j2);
                                 auto r = unit_cell().get_cartesian_coordinates(v).length();
                                 if (r < Rmt[unit_cell().atom(ia).type_id()]) {
                                     auto ir = fft_grid_.index_by_coord(j0, j1, j2 - z_off);
@@ -1442,7 +1406,7 @@ Simulation_context::init_step_function()
     });
 
     theta_    = sddk::mdarray<double, 1>(spfft<double>().local_slice_size());
-    theta_pw_ = sddk::mdarray<double_complex, 1>(gvec().num_gvec());
+    theta_pw_ = sddk::mdarray<std::complex<double>, 1>(gvec().num_gvec());
 
     try {
         for (int ig = 0; ig < gvec().num_gvec(); ig++) {
@@ -1450,16 +1414,16 @@ Simulation_context::init_step_function()
         }
         theta_pw_[0] += 1.0;
 
-        std::vector<double_complex> ftmp(gvec_fft().gvec_count_fft());
+        std::vector<std::complex<double>> ftmp(gvec_fft().count());
         this->gvec_fft().scatter_pw_global(&theta_pw_[0], &ftmp[0]);
         spfft<double>().backward(reinterpret_cast<double const*>(ftmp.data()), SPFFT_PU_HOST);
         double* theta_ptr = spfft<double>().local_slice_size() == 0 ? nullptr : &theta_[0];
-        spfft_output(spfft<double>(), theta_ptr);
+        fft::spfft_output(spfft<double>(), theta_ptr);
     } catch (...) {
         std::stringstream s;
         s << "fft_grid = " << fft_grid_[0] << " " << fft_grid_[1] << " " << fft_grid_[2] << std::endl
           << "spfft<double>().local_slice_size() = " << spfft<double>().local_slice_size() << std::endl
-          << "gvec_fft().gvec_count_fft() = " << gvec_fft().gvec_count_fft();
+          << "gvec_fft().count() = " << gvec_fft().count();
         RTE_THROW(s);
     }
 
@@ -1468,7 +1432,7 @@ Simulation_context::init_step_function()
         vit += theta_[i];
     }
     vit *= (unit_cell().omega() / fft_grid().num_points());
-    sddk::Communicator(spfft<double>().communicator()).allreduce(&vit, 1);
+    mpi::Communicator(spfft<double>().communicator()).allreduce(&vit, 1);
 
     if (std::abs(vit - unit_cell().volume_it()) > 1e-10) {
         std::stringstream s;
@@ -1479,9 +1443,9 @@ Simulation_context::init_step_function()
         }
     }
     if (cfg().control().print_checksum()) {
-        double_complex z1 = theta_pw_.checksum();
-        double d1         = theta_.checksum();
-        sddk::Communicator(spfft<double>().communicator()).allreduce(&d1, 1);
+        auto z1 = theta_pw_.checksum();
+        auto d1 = theta_.checksum();
+        mpi::Communicator(spfft<double>().communicator()).allreduce(&d1, 1);
         utils::print_checksum("theta", d1, this->out());
         utils::print_checksum("theta_pw", z1, this->out());
     }
@@ -1522,19 +1486,17 @@ Simulation_context::init_comm()
     }
 
     /* setup MPI grid */
-    mpi_grid_ = std::make_unique<sddk::MPI_grid>(std::vector<int>({npc, npr}), comm_band_);
+    mpi_grid_ = std::make_unique<mpi::Grid>(std::vector<int>({npr, npc}), comm_band_);
 
     /* here we know the number of ranks for band parallelization */
 
     /* if we have multiple ranks per node and band parallelization, switch to parallel FFT for coarse mesh */
-    if ((npr == npb) || (sddk::num_ranks_per_node() > acc::num_devices() && comm_band().size() > 1)) {
+    if ((npr == npb) || (mpi::num_ranks_per_node() > acc::num_devices() && comm_band().size() > 1)) {
         cfg().control().fft_mode("parallel");
     }
 
     /* create communicator, orthogonal to comm_fft_coarse */
     comm_ortho_fft_coarse_ = comm().split(comm_fft_coarse().rank());
-
-    /* create communicator, orthogonal to comm_fft_coarse within a band communicator */
-    comm_band_ortho_fft_coarse_ = comm_band().split(comm_fft_coarse().rank());
 }
+
 } // namespace sirius
