@@ -148,10 +148,10 @@ Hamiltonian_k<T>::get_h_o_diag_pw() const
             int offs = uc.atom_type(iat).offset_lo();
 
             if (what & 1) {
-                sddk::linalg(sddk::linalg_t::blas)
-                    .gemm('N', 'N', kp_.num_gkvec_loc(), nbf, nbf, &sddk::linalg_const<std::complex<T>>::one(),
+                la::wrap(la::lib_t::blas)
+                    .gemm('N', 'N', kp_.num_gkvec_loc(), nbf, nbf, &la::constant<std::complex<T>>::one(),
                           &beta_gk_t(0, offs), beta_gk_t.ld(), &d_sum(0, 0), d_sum.ld(),
-                          &sddk::linalg_const<std::complex<T>>::zero(), &beta_gk_tmp(0, 0), beta_gk_tmp.ld());
+                          &la::constant<std::complex<T>>::zero(), &beta_gk_tmp(0, 0), beta_gk_tmp.ld());
                 #pragma omp parallel
                 for (int xi = 0; xi < nbf; xi++) {
                     #pragma omp for schedule(static) nowait
@@ -164,10 +164,10 @@ Hamiltonian_k<T>::get_h_o_diag_pw() const
             }
 
             if (what & 2) {
-                sddk::linalg(sddk::linalg_t::blas)
-                    .gemm('N', 'N', kp_.num_gkvec_loc(), nbf, nbf, &sddk::linalg_const<std::complex<T>>::one(),
+                la::wrap(la::lib_t::blas)
+                    .gemm('N', 'N', kp_.num_gkvec_loc(), nbf, nbf, &la::constant<std::complex<T>>::one(),
                           &beta_gk_t(0, offs), beta_gk_t.ld(), &q_sum(0, 0), q_sum.ld(),
-                          &sddk::linalg_const<std::complex<T>>::zero(), &beta_gk_tmp(0, 0), beta_gk_tmp.ld());
+                          &la::constant<std::complex<T>>::zero(), &beta_gk_tmp(0, 0), beta_gk_tmp.ld());
                 #pragma omp parallel
                 for (int xi = 0; xi < nbf; xi++) {
                     #pragma omp for schedule(static) nowait
@@ -304,7 +304,7 @@ Hamiltonian_k<T>::get_h_o_diag_lapw() const
 
 template <typename T>
 void
-Hamiltonian_k<T>::set_fv_h_o(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatrix<std::complex<T>>& o__) const
+Hamiltonian_k<T>::set_fv_h_o(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const
 {
     PROFILE("sirius::Hamiltonian_k::set_fv_h_o");
 
@@ -325,20 +325,20 @@ Hamiltonian_k<T>::set_fv_h_o(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatrix<
     /* current processing unit */
     auto pu = H0_.ctx().processing_unit();
 
-    auto la  = sddk::linalg_t::none;
+    auto la  = la::lib_t::none;
     auto mt  = sddk::memory_t::none;
     auto mt1 = sddk::memory_t::none;
     int nb   = 0;
     switch (pu) {
         case sddk::device_t::CPU: {
-            la  = sddk::linalg_t::blas;
+            la  = la::lib_t::blas;
             mt  = sddk::memory_t::host;
             mt1 = sddk::memory_t::host;
             nb  = 1;
             break;
         }
         case sddk::device_t::GPU: {
-            la  = sddk::linalg_t::spla;
+            la  = la::lib_t::spla;
             mt  = sddk::memory_t::host_pinned;
             mt1 = sddk::memory_t::device;
             nb  = 1;
@@ -482,14 +482,14 @@ Hamiltonian_k<T>::set_fv_h_o(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatrix<
             utils::print_checksum("halm_col", z3, H0_.ctx().out());
         }
 
-        sddk::linalg(la).gemm('N', 'T', kp.num_gkvec_row(), kp.num_gkvec_col(), num_mt_aw,
-                        &sddk::linalg_const<std::complex<T>>::one(), alm_row.at(mt1, 0, 0, s), alm_row.ld(),
-                        alm_col.at(mt1, 0, 0, s), alm_col.ld(), &sddk::linalg_const<std::complex<T>>::one(), o__.at(mt),
+        la::wrap(la).gemm('N', 'T', kp.num_gkvec_row(), kp.num_gkvec_col(), num_mt_aw,
+                        &la::constant<std::complex<T>>::one(), alm_row.at(mt1, 0, 0, s), alm_row.ld(),
+                        alm_col.at(mt1, 0, 0, s), alm_col.ld(), &la::constant<std::complex<T>>::one(), o__.at(mt),
                         o__.ld());
 
-        sddk::linalg(la).gemm('N', 'T', kp.num_gkvec_row(), kp.num_gkvec_col(), num_mt_aw,
-                        &sddk::linalg_const<std::complex<T>>::one(), alm_row.at(mt1, 0, 0, s), alm_row.ld(),
-                        halm_col.at(mt1, 0, 0, s), halm_col.ld(), &sddk::linalg_const<std::complex<T>>::one(), h__.at(mt),
+        la::wrap(la).gemm('N', 'T', kp.num_gkvec_row(), kp.num_gkvec_col(), num_mt_aw,
+                        &la::constant<std::complex<T>>::one(), alm_row.at(mt1, 0, 0, s), alm_row.ld(),
+                        halm_col.at(mt1, 0, 0, s), halm_col.ld(), &la::constant<std::complex<T>>::one(), h__.at(mt),
                         h__.ld());
     }
 
@@ -620,7 +620,7 @@ Hamiltonian_k<T>::set_fv_h_o_apw_lo(Atom const& atom__, int ia__, sddk::mdarray<
 
 template <typename T>
 void
-Hamiltonian_k<T>::set_fv_h_o_lo_lo(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatrix<std::complex<T>>& o__) const
+Hamiltonian_k<T>::set_fv_h_o_lo_lo(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const
 {
     PROFILE("sirius::Hamiltonian_k::set_fv_h_o_lo_lo");
 
@@ -664,7 +664,7 @@ Hamiltonian_k<T>::set_fv_h_o_lo_lo(sddk::dmatrix<std::complex<T>>& h__, sddk::dm
 
 template <typename T>
 void
-Hamiltonian_k<T>::set_fv_h_o_it(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatrix<std::complex<T>>& o__) const
+Hamiltonian_k<T>::set_fv_h_o_it(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const
 {
     PROFILE("sirius::Hamiltonian_k::set_fv_h_o_it");
 
@@ -685,7 +685,7 @@ Hamiltonian_k<T>::set_fv_h_o_it(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatr
             auto gkvec_row_cart = kp.gkvec().template gkvec_cart<sddk::index_domain_t::global>(ig_row);
             int ig12            = H0().ctx().gvec().index_g12(gvec_row, gvec_col);
             /* pw kinetic energy */
-            double t1 = 0.5 * geometry3d::dot(gkvec_row_cart, gkvec_col_cart);
+            double t1 = 0.5 * r3::dot(gkvec_row_cart, gkvec_col_cart);
 
             h__(igk_row, igk_col) += H0().potential().veff_pw(ig12);
             o__(igk_row, igk_col) += H0().ctx().theta_pw(ig12);
@@ -713,7 +713,7 @@ Hamiltonian_k<T>::set_fv_h_o_it(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatr
 }
 
 //== template <spin_block_t sblock>
-//== void Band::apply_uj_correction(mdarray<double_complex, 2>& fv_states, mdarray<double_complex, 3>& hpsi)
+//== void Band::apply_uj_correction(mdarray<std::complex<double>, 2>& fv_states, mdarray<std::complex<double>, 3>& hpsi)
 //== {
 //==     Timer t("sirius::Band::apply_uj_correction");
 //==
@@ -743,7 +743,7 @@ Hamiltonian_k<T>::set_fv_h_o_it(sddk::dmatrix<std::complex<T>>& h__, sddk::dmatr
 //==                             for (int lm1 = Utils::lm_by_l_m(l, -l); lm1 <= Utils::lm_by_l_m(l, l); lm1++)
 //==                             {
 //==                                 int idx1 = type->indexb_by_lm_order(lm1, order1);
-//==                                 double_complex z1 = fv_states(offset + idx1, ist) * ori;
+//==                                 std::complex<double> z1 = fv_states(offset + idx1, ist) * ori;
 //==
 //==                                 if (sblock == uu)
 //==                                 {
@@ -795,7 +795,7 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
 
     auto pu = ctx.processing_unit();
 
-    auto la  = (pu == sddk::device_t::CPU) ? sddk::linalg_t::blas : sddk::linalg_t::gpublas;
+    auto la  = (pu == sddk::device_t::CPU) ? la::lib_t::blas : la::lib_t::gpublas;
     auto mem = (pu == sddk::device_t::CPU) ? sddk::memory_t::host : sddk::memory_t::device;
 
     auto pp = env::print_performance();
@@ -822,18 +822,6 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
 
     auto& comm = kp().comm();
 
-    //if (!apw_only__) {
-    //    if (hphi__ != nullptr) {
-    //        /* zero the local-orbital part */
-    //        hphi__->mt_coeffs(0).zero(mem, N__, n__);
-    //    }
-    //    if (ophi__ != nullptr) {
-    //        /* zero the local-orbital part */
-    //        ophi__->mt_coeffs(0).zero(sddk::memory_t::host, N__, n__);
-    //        ophi__->mt_coeffs(0).zero(mem, N__, n__);
-    //    }
-    //}
-
     /* ophi is computed on the CPU to avoid complicated GPU implementation */
     if (is_device_memory(mem)) {
         phi__.copy_mt_to(sddk::memory_t::host, wf::spin_index(0), b__);
@@ -848,13 +836,17 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
 
     if (!phi_is_lo__) {
         /* interstitial part */
-        H0_.local_op().apply_fplapw(reinterpret_cast<spfft_transform_type<T>&>(kp().spfft_transform()),
+        H0_.local_op().apply_fplapw(reinterpret_cast<fft::spfft_transform_type<T>&>(kp().spfft_transform()),
                 kp().gkvec_fft_sptr(), b__, phi__, hphi__, ophi__, nullptr, nullptr);
 
         if (pcs) {
             if (hphi__) {
                 auto cs = hphi__->checksum(mem, wf::spin_index(0), b__);
+                auto cs_pw = hphi__->checksum_pw(mem, wf::spin_index(0), b__);
+                auto cs_mt = hphi__->checksum_mt(mem, wf::spin_index(0), b__);
                 if (comm.rank() == 0) {
+                    utils::print_checksum("hloc_phi_pw", cs_pw, RTE_OUT(std::cout));
+                    utils::print_checksum("hloc_phi_mt", cs_mt, RTE_OUT(std::cout));
                     utils::print_checksum("hloc_phi", cs, RTE_OUT(std::cout));
                 }
             }
@@ -883,8 +875,8 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
     /* block size of scalapack distribution */
     int bs = ctx.cyclic_block_size();
 
-    auto& one = sddk::linalg_const<Tc>::one();
-    auto& zero = sddk::linalg_const<Tc>::zero();
+    auto& one = la::constant<Tc>::one();
+    auto& zero = la::constant<Tc>::zero();
 
     /* apply APW-lo part of Hamiltonian to lo- part of wave-functions */
     auto apply_hmt_apw_lo = [this, &ctx, &phi__, la, mem, &b__, &spl_atoms](wf::Wave_functions_mt<T>& h_apw_lo__)
@@ -902,11 +894,11 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
 
             auto& hmt = this->H0_.hmt(ia);
 
-            sddk::linalg(la).gemm('N', 'N', naw, b__.size(), nlo,
-                    &sddk::linalg_const<Tc>::one(),
+            la::wrap(la).gemm('N', 'N', naw, b__.size(), nlo,
+                    &la::constant<Tc>::one(),
                     hmt.at(mem, 0, naw), hmt.ld(),
                     phi__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(b__.begin())), phi__.ld(),
-                    &sddk::linalg_const<Tc>::zero(),
+                    &la::constant<Tc>::zero(),
                     h_apw_lo__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(0)), h_apw_lo__.ld(),
                     stream_id(tid));
         }
@@ -964,10 +956,10 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
 
             auto& hmt = H0_.hmt(ia);
 
-            sddk::linalg(la).gemm('N', 'N', nlo, b__.size(), nlo, &sddk::linalg_const<Tc>::one(),
+            la::wrap(la).gemm('N', 'N', nlo, b__.size(), nlo, &la::constant<Tc>::one(),
                             hmt.at(mem, naw, naw), hmt.ld(),
                             phi__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(b__.begin())), phi__.ld(),
-                            &sddk::linalg_const<Tc>::one(),
+                            &la::constant<Tc>::one(),
                             hphi__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(b__.begin())), hphi__.ld(),
                             stream_id(tid));
         }
@@ -1024,10 +1016,10 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
             auto& hmt = H0_.hmt(ia);
 
             // TODO: use in-place trmm
-            sddk::linalg(la).gemm('N', 'N', naw, b__.size(), naw,
-                    &sddk::linalg_const<Tc>::one(), hmt.at(mem), hmt.ld(),
+            la::wrap(la).gemm('N', 'N', naw, b__.size(), naw,
+                    &la::constant<Tc>::one(), hmt.at(mem), hmt.ld(),
                     alm_phi__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(0)), alm_phi__.ld(),
-                    &sddk::linalg_const<Tc>::zero(),
+                    &la::constant<Tc>::zero(),
                     halm_phi__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(0)), halm_phi__.ld(),
                     stream_id(tid));
         }
@@ -1050,10 +1042,10 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
 
             // TODO: add stream_id
 
-            sddk::linalg(la).gemm('N', 'N', nlo, b__.size(), naw, &sddk::linalg_const<Tc>::one(),
+            la::wrap(la).gemm('N', 'N', nlo, b__.size(), naw, &la::constant<Tc>::one(),
                     hmt.at(mem, naw, 0), hmt.ld(),
                     alm_phi__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(0)), alm_phi__.ld(),
-                    &sddk::linalg_const<Tc>::one(),
+                    &la::constant<Tc>::one(),
                     hphi__.at(mem, 0, aidx, wf::spin_index(0), wf::band_index(b__.begin())),
                     hphi__.ld(), stream_id(tid));
         }
@@ -1135,8 +1127,8 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
      * |      |
      * +------+
      */
-    sddk::dmatrix<Tc> h_apw_lo_phi_lo;
-    sddk::dmatrix<Tc> o_apw_lo_phi_lo;
+    la::dmatrix<Tc> h_apw_lo_phi_lo;
+    la::dmatrix<Tc> o_apw_lo_phi_lo;
 
     std::vector<int> num_mt_apw_coeffs(ctx.unit_cell().num_atoms());
     for (int ia = 0; ia < ctx.unit_cell().num_atoms(); ia++) {
@@ -1151,19 +1143,19 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
         if (hphi__) {
             apply_hmt_apw_lo(tmp);
 
-            h_apw_lo_phi_lo = sddk::dmatrix<Tc>(ctx.unit_cell().mt_aw_basis_size(), b__.size(),
+            h_apw_lo_phi_lo = la::dmatrix<Tc>(ctx.unit_cell().mt_aw_basis_size(), b__.size(),
                                                 ctx.blacs_grid(), bs, bs);
 
             auto layout_in = tmp.grid_layout_mt(wf::spin_index(0), wf::band_range(0, b__.size()));
-            costa::transform(layout_in, h_apw_lo_phi_lo.grid_layout(), 'N', one, zero, comm.mpi_comm());
+            costa::transform(layout_in, h_apw_lo_phi_lo.grid_layout(), 'N', one, zero, comm.native());
         }
         if (ophi__) {
             apply_omt_apw_lo(tmp);
-            o_apw_lo_phi_lo = sddk::dmatrix<Tc>(ctx.unit_cell().mt_aw_basis_size(), b__.size(),
+            o_apw_lo_phi_lo = la::dmatrix<Tc>(ctx.unit_cell().mt_aw_basis_size(), b__.size(),
                                                 ctx.blacs_grid(), bs, bs);
 
             auto layout_in = tmp.grid_layout_mt(wf::spin_index(0), wf::band_range(0, b__.size()));
-            costa::transform(layout_in, o_apw_lo_phi_lo.grid_layout(), 'N', one, zero, comm.mpi_comm());
+            costa::transform(layout_in, o_apw_lo_phi_lo.grid_layout(), 'N', one, zero, comm.native());
         }
     }
 
@@ -1191,7 +1183,7 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
     }
 
     /* <A_{lm}^{\alpha}(G) | C_j(G) > for all Alm matching coefficients */
-    sddk::dmatrix<Tc> alm_phi(ctx.unit_cell().mt_aw_basis_size(), b__.size(), ctx.blacs_grid(), bs, bs);
+    la::dmatrix<Tc> alm_phi(ctx.unit_cell().mt_aw_basis_size(), b__.size(), ctx.blacs_grid(), bs, bs);
 
     /*  compute APW-APW contribution
      *                         n
@@ -1263,11 +1255,11 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
                 wf::Wave_functions_mt<T> halm_phi_slab(comm, num_mt_apw_coeffs_in_block, wf::num_mag_dims(0),
                         wf::num_bands(b__.size()), sddk::memory_t::host);
 
-                sddk::dmatrix<Tc> halm_phi(num_mt_aw, b__.size(), ctx.blacs_grid(), bs, bs);
+                la::dmatrix<Tc> halm_phi(num_mt_aw, b__.size(), ctx.blacs_grid(), bs, bs);
                 {
                     auto layout_in = alm_phi.grid_layout(offset_aw_global, 0, num_mt_aw, b__.size());
                     auto layout_out = alm_phi_slab.grid_layout_mt(wf::spin_index(0), wf::band_range(0, b__.size()));
-                    costa::transform(layout_in, layout_out, 'N', one, zero, comm.mpi_comm());
+                    costa::transform(layout_in, layout_out, 'N', one, zero, comm.native());
                 }
 
                 {
@@ -1279,7 +1271,7 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
                 {
                     auto layout_in = halm_phi_slab.grid_layout_mt(wf::spin_index(0), wf::band_range(0, b__.size()));
                     auto layout_out = halm_phi.grid_layout();
-                    costa::transform(layout_in, layout_out, 'N', one, zero, comm.mpi_comm());
+                    costa::transform(layout_in, layout_out, 'N', one, zero, comm.native());
                 }
 
                 /* APW-APW contribution to hphi */
@@ -1355,7 +1347,7 @@ Hamiltonian_k<T>::apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range
         {
             auto layout_in = alm_phi.grid_layout();
             auto layout_out = tmp.grid_layout_mt(wf::spin_index(0), wf::band_range(0, b__.size()));
-            costa::transform(layout_in, layout_out, 'N', one, zero, comm.mpi_comm());
+            costa::transform(layout_in, layout_out, 'N', one, zero, comm.native());
         }
 
         if (ophi__) {
@@ -1406,7 +1398,7 @@ Hamiltonian_k<T>::apply_b(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_fun
     int nfv = H0().ctx().num_fv_states();
 
     auto bxypsi = bpsi__.size() == 2 ? nullptr : &bpsi__[2];
-    H0().local_op().apply_fplapw(reinterpret_cast<spfft_transform_type<T>&>(kp().spfft_transform()),
+    H0().local_op().apply_fplapw(reinterpret_cast<fft::spfft_transform_type<T>&>(kp().spfft_transform()),
             this->kp().gkvec_fft_sptr(), wf::band_range(0, nfv), psi__, nullptr, nullptr, &bpsi__[0], bxypsi);
     H0().apply_bmt(psi__, bpsi__);
 
@@ -1444,13 +1436,13 @@ template std::pair<sddk::mdarray<double, 2>, sddk::mdarray<double, 2>>
 Hamiltonian_k<double>::get_h_o_diag_pw<double, 3>() const;
 
 template std::pair<sddk::mdarray<double, 2>, sddk::mdarray<double, 2>>
-Hamiltonian_k<double>::get_h_o_diag_pw<double_complex, 1>() const;
+Hamiltonian_k<double>::get_h_o_diag_pw<std::complex<double>, 1>() const;
 
 template std::pair<sddk::mdarray<double, 2>, sddk::mdarray<double, 2>>
-Hamiltonian_k<double>::get_h_o_diag_pw<double_complex, 2>() const;
+Hamiltonian_k<double>::get_h_o_diag_pw<std::complex<double>, 2>() const;
 
 template std::pair<sddk::mdarray<double, 2>, sddk::mdarray<double, 2>>
-Hamiltonian_k<double>::get_h_o_diag_pw<double_complex, 3>() const;
+Hamiltonian_k<double>::get_h_o_diag_pw<std::complex<double>, 3>() const;
 
 template std::pair<sddk::mdarray<double, 2>, sddk::mdarray<double, 2>>
 Hamiltonian_k<double>::get_h_o_diag_lapw<1>() const;
