@@ -26,7 +26,7 @@
 #define __LOCAL_OPERATOR_HPP__
 
 #include "SDDK/memory.hpp"
-#include "SDDK/fft.hpp"
+#include "fft/fft.hpp"
 #include "typedefs.hpp"
 #include "utils/rte.hpp"
 
@@ -37,8 +37,7 @@ class Simulation_context;
 template <typename T>
 class Smooth_periodic_function;
 }
-namespace sddk {
-class FFT3D;
+namespace fft {
 class Gvec_fft;
 }
 namespace wf {
@@ -212,10 +211,10 @@ class Local_operator
     Simulation_context const& ctx_;
 
     /// Coarse-grid FFT driver for this operator.
-    spfft_transform_type<T>& fft_coarse_;
+    fft::spfft_transform_type<T>& fft_coarse_;
 
     /// Distribution of the G-vectors for the FFT transformation.
-    std::shared_ptr<sddk::Gvec_fft> gvec_coarse_p_;
+    std::shared_ptr<fft::Gvec_fft> gvec_coarse_p_;
 
     /// Kinetic energy of G+k plane-waves.
     sddk::mdarray<T, 1> pw_ekin_;
@@ -267,12 +266,12 @@ class Local_operator
      *  \param [in] potential     Effective potential and magnetic fields \f$ V_{eff}({\bf r}) \f$ and
      *                             \f$ {\bf B}_{eff}({\bf r}) \f$ on the fine FFT grid.
      */
-    Local_operator(Simulation_context const& ctx__, spfft_transform_type<T>& fft_coarse__,
-                   std::shared_ptr<sddk::Gvec_fft> gvec_coarse_fft__, Potential* potential__ = nullptr);
+    Local_operator(Simulation_context const& ctx__, fft::spfft_transform_type<T>& fft_coarse__,
+                   std::shared_ptr<fft::Gvec_fft> gvec_coarse_fft__, Potential* potential__ = nullptr);
 
     /// Prepare the k-point dependent arrays.
     /** \param [in] gkvec_p  FFT-friendly G+k vector partitioning. */
-    void prepare_k(sddk::Gvec_fft const& gkvec_p__);
+    void prepare_k(fft::Gvec_fft const& gkvec_p__);
 
     /// Apply local part of Hamiltonian to pseudopotential wave-functions.
     /** \param [in]  spfftk  SpFFT transform object for G+k vectors.
@@ -290,7 +289,7 @@ class Local_operator
      *
      *  Local Hamiltonian includes kinetic term and local part of potential.
      */
-    void apply_h(spfft_transform_type<T>& spfftk__, std::shared_ptr<sddk::Gvec_fft> gkvec_fft__,
+    void apply_h(fft::spfft_transform_type<T>& spfftk__, std::shared_ptr<fft::Gvec_fft> gkvec_fft__,
             wf::spin_range spins__, wf::Wave_functions<T> const& phi__, wf::Wave_functions<T>& hphi__,
             wf::band_range br__);
 
@@ -305,7 +304,7 @@ class Local_operator
      *
      *  Only plane-wave part of output wave-functions is changed.
      */
-    void apply_fplapw(spfft_transform_type<T>& spfftik__, std::shared_ptr<sddk::Gvec_fft> gkvec_fft__,
+    void apply_fplapw(fft::spfft_transform_type<T>& spfftik__, std::shared_ptr<fft::Gvec_fft> gkvec_fft__,
             wf::band_range b__, wf::Wave_functions<T>& phi__, wf::Wave_functions<T>* hphi__,
             wf::Wave_functions<T>* ophi__, wf::Wave_functions<T>* bzphi__, wf::Wave_functions<T>* bxyphi__);
 
