@@ -22,8 +22,6 @@
  *  \brief Contains implementation of sirius::Non_local_operator class.
  */
 
-#include "SDDK/wf_inner.hpp"
-#include "SDDK/wf_trans.hpp"
 #include "non_local_operator.hpp"
 #include "hubbard/hubbard_matrix.hpp"
 
@@ -46,11 +44,11 @@ Non_local_operator<T>::Non_local_operator(Simulation_context const& ctx__)
     }
 
     switch (pu_) {
-        case device_t::GPU: {
-            packed_mtrx_offset_.allocate(memory_t::device).copy_to(memory_t::device);
+        case sddk::device_t::GPU: {
+            packed_mtrx_offset_.allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
             break;
         }
-        case device_t::CPU: {
+        case sddk::device_t::CPU: {
             break;
         }
     }
@@ -61,9 +59,9 @@ D_operator<T>::D_operator(Simulation_context const& ctx_)
     : Non_local_operator<T>(ctx_)
 {
     if (ctx_.gamma_point()) {
-        this->op_ = mdarray<T, 3>(1, this->packed_mtrx_size_, ctx_.num_mag_dims() + 1);
+        this->op_ = sddk::mdarray<T, 3>(1, this->packed_mtrx_size_, ctx_.num_mag_dims() + 1);
     } else {
-        this->op_ = mdarray<T, 3>(2, this->packed_mtrx_size_, ctx_.num_mag_dims() + 1);
+        this->op_ = sddk::mdarray<T, 3>(2, this->packed_mtrx_size_, ctx_.num_mag_dims() + 1);
     }
     this->op_.zero();
     initialize();
@@ -87,7 +85,7 @@ D_operator<T>::initialize()
 
         /* in case of spin orbit coupling */
         if (uc.atom(ia).type().spin_orbit_coupling()) {
-            mdarray<std::complex<T>, 3> d_mtrx_so(nbf, nbf, 4);
+            sddk::mdarray<std::complex<T>, 3> d_mtrx_so(nbf, nbf, 4);
             d_mtrx_so.zero();
 
             /* transform the d_mtrx */
@@ -222,13 +220,13 @@ D_operator<T>::initialize()
         }
     }
 
-    if (this->ctx_.print_checksum() && this->ctx_.comm().rank() == 0) {
+    if (this->ctx_.print_checksum()) {
         auto cs = this->op_.checksum();
-        utils::print_checksum("D_operator", cs);
+        utils::print_checksum("D_operator", cs, this->ctx_.out());
     }
 
-    if (this->pu_ == device_t::GPU && uc.mt_lo_basis_size() != 0) {
-        this->op_.allocate(memory_t::device).copy_to(memory_t::device);
+    if (this->pu_ == sddk::device_t::GPU && uc.mt_lo_basis_size() != 0) {
+        this->op_.allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
     }
 
     /* D-operator is not diagonal in spin in case of non-collinear magnetism
@@ -245,9 +243,9 @@ Q_operator<T>::Q_operator(Simulation_context const& ctx__)
     /* Q-operator is independent of spin if there is no spin-orbit; however, it simplifies the apply()
      * method if the Q-operator has a spin index */
     if (this->ctx_.gamma_point()) {
-        this->op_ = mdarray<T, 3>(1, this->packed_mtrx_size_, this->ctx_.num_mag_dims() + 1);
+        this->op_ = sddk::mdarray<T, 3>(1, this->packed_mtrx_size_, this->ctx_.num_mag_dims() + 1);
     } else {
-        this->op_ = mdarray<T, 3>(2, this->packed_mtrx_size_, this->ctx_.num_mag_dims() + 1);
+        this->op_ = sddk::mdarray<T, 3>(2, this->packed_mtrx_size_, this->ctx_.num_mag_dims() + 1);
     }
     this->op_.zero();
     initialize();
@@ -285,7 +283,7 @@ Q_operator<T>::initialize()
                                         /* The F coefficients are already "block diagonal" so we do a full
                                            summation. We actually rotate the q_matrices only */
                                         if (uc.atom(ia).type().compare_index_beta_functions(xi1, xi1p)) {
-                                            result += this->ctx_.augmentation_op(iat)->q_mtrx(xi1p, xi2p) *
+                                            result += this->ctx_.augmentation_op(iat).q_mtrx(xi1p, xi2p) *
                                                       (uc.atom(ia).type().f_coefficients(xi1, xi1p, sj, 0) *
                                                            uc.atom(ia).type().f_coefficients(xi2p, xi2, 0, si) +
                                                        uc.atom(ia).type().f_coefficients(xi1, xi1p, sj, 1) *
@@ -309,19 +307,19 @@ Q_operator<T>::initialize()
                 } else {
                     for (int ispn = 0; ispn < this->ctx_.num_spins(); ispn++) {
                         this->op_(0, this->packed_mtrx_offset_(ia) + xi2 * nbf + xi1, ispn) =
-                            this->ctx_.augmentation_op(iat)->q_mtrx(xi1, xi2);
+                            this->ctx_.augmentation_op(iat).q_mtrx(xi1, xi2);
                     }
                 }
             }
         }
     }
-    if (this->ctx_.print_checksum() && this->ctx_.comm().rank() == 0) {
+    if (this->ctx_.print_checksum()) {
         auto cs = this->op_.checksum();
-        utils::print_checksum("Q_operator", cs);
+        utils::print_checksum("Q_operator", cs, this->ctx_.out());
     }
 
-    if (this->pu_ == device_t::GPU && uc.mt_lo_basis_size() != 0) {
-        this->op_.allocate(memory_t::device).copy_to(memory_t::device);
+    if (this->pu_ == sddk::device_t::GPU && uc.mt_lo_basis_size() != 0) {
+        this->op_.allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
     }
 
     this->is_null_ = true;
@@ -336,195 +334,17 @@ Q_operator<T>::initialize()
     }
 }
 
-template <typename T>
-void
-apply_non_local_d_q(spin_range spins__, int N__, int n__, Beta_projectors<real_type<T>>& beta__,
-                    Wave_functions<real_type<T>>& phi__, D_operator<real_type<T>>* d_op__,
-                    Wave_functions<real_type<T>>* hphi__, Q_operator<real_type<T>>* q_op__,
-                    Wave_functions<real_type<T>>* sphi__)
-{
-
-    for (int i = 0; i < beta__.num_chunks(); i++) {
-        /* generate beta-projectors for a block of atoms */
-        beta__.generate(i);
-
-        for (int ispn : spins__) {
-            auto beta_phi = beta__.template inner<T>(i, phi__, ispn, N__, n__);
-
-            if (hphi__ && d_op__) {
-                /* apply diagonal spin blocks */
-                d_op__->apply(i, ispn, *hphi__, N__, n__, beta__, beta_phi);
-                if (!d_op__->is_diag() && hphi__->num_sc() == 2) {
-                    /* apply non-diagonal spin blocks */
-                    /* xor 3 operator will map 0 to 3 and 1 to 2 */
-                    d_op__->apply(i, ispn ^ 3, *hphi__, N__, n__, beta__, beta_phi);
-                }
-            }
-
-            if (sphi__ && q_op__) {
-                /* apply Q operator (diagonal in spin) */
-                q_op__->apply(i, ispn, *sphi__, N__, n__, beta__, beta_phi);
-                if (!q_op__->is_diag() && sphi__->num_sc() == 2) {
-                    q_op__->apply(i, ispn ^ 3, *sphi__, N__, n__, beta__, beta_phi);
-                }
-            }
-        }
-    }
-}
-
-/// Compute |sphi> = (1 + Q)|phi>
-template <typename T>
-void
-apply_S_operator(device_t pu__, spin_range spins__, int N__, int n__, Beta_projectors<real_type<T>>& beta__,
-                 Wave_functions<real_type<T>>& phi__, Q_operator<real_type<T>>* q_op__,
-                 Wave_functions<real_type<T>>& sphi__)
-{
-    for (auto s : spins__) {
-        sphi__.copy_from(pu__, n__, phi__, s, N__, s, N__);
-    }
-
-    if (q_op__) {
-        apply_non_local_d_q<T>(spins__, N__, n__, beta__, phi__, nullptr, nullptr, q_op__, &sphi__);
-    }
-}
-
-template <typename T>
-void
-apply_U_operator(Simulation_context& ctx__, spin_range spins__, int N__, int n__, Wave_functions<T>& hub_wf__,
-                 Wave_functions<T>& phi__, U_operator<T>& um__, Wave_functions<T>& hphi__)
-{
-    if (!ctx__.hubbard_correction()) {
-        return;
-    }
-
-    dmatrix<std::complex<T>> dm(hub_wf__.num_wf(), n__);
-    if (ctx__.processing_unit() == device_t::GPU) {
-        dm.allocate(memory_t::device);
-    }
-
-    auto la = linalg_t::none;
-    auto mt = memory_t::none;
-    switch (ctx__.processing_unit()) {
-        case device_t::CPU: {
-            la = linalg_t::blas;
-            mt = memory_t::host;
-            break;
-        }
-        case device_t::GPU: {
-            la = linalg_t::gpublas;
-            mt = memory_t::device;
-            break;
-        }
-        default:
-            break;
-    }
-    /* First calculate the local part of the projections
-       dm(i, n) = <phi_i| S |psi_{nk}> */
-    sddk::inner(ctx__.spla_context(), spins__, hub_wf__, 0, hub_wf__.num_wf(), phi__, N__, n__, dm, 0, 0);
-
-    dmatrix<std::complex<T>> Up(hub_wf__.num_wf(), n__);
-    if (ctx__.processing_unit() == device_t::GPU) {
-        Up.allocate(memory_t::device);
-    }
-
-    if (ctx__.num_mag_dims() == 3) {
-        Up.zero();
-        #pragma omp parallel for schedule(static)
-        for (int at_lvl = 0; at_lvl < (int)um__.atomic_orbitals().size(); at_lvl++) {
-            const int ia     = um__.atomic_orbitals(at_lvl).first;
-            auto const& atom = ctx__.unit_cell().atom(ia);
-            if (atom.type().lo_descriptor_hub(um__.atomic_orbitals(at_lvl).second).use_for_calculation()) {
-                const int lmax_at = 2 * atom.type().lo_descriptor_hub(um__.atomic_orbitals(at_lvl).second).l() + 1;
-                // we apply the hubbard correction. For now I have no papers
-                // giving me the formula for the SO case so I rely on QE for it
-                // but I do not like it at all
-                for (int s1 = 0; s1 < ctx__.num_spins(); s1++) {
-                    for (int s2 = 0; s2 < ctx__.num_spins(); s2++) {
-                      // TODO: replace this with matrix matrix multiplication
-                        for (int nbd = 0; nbd < n__; nbd++) {
-                            for (int m1 = 0; m1 < lmax_at; m1++) {
-                                for (int m2 = 0; m2 < lmax_at; m2++) {
-                                    const int ind = (s1 == s2) * s1 + (1 + 2 * s2 + s1) * (s1 != s2);
-                                    Up(um__.nhwf() * s1 + um__.offset(at_lvl) + m1, nbd) +=
-                                      um__(um__.offset(at_lvl) + m2, um__.offset(at_lvl) + m1, ind) *
-                                      dm(um__.nhwf() * s2 + um__.offset(at_lvl) + m2, nbd);
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-        }
-        if (ctx__.processing_unit() == device_t::GPU) {
-            Up.copy_to(memory_t::device);
-        }
-    } else {
-        linalg(la).gemm('N', 'N', um__.nhwf(), n__, um__.nhwf(), &linalg_const<double_complex>::one(),
-                        um__.at(mt, 0, 0, spins__()), um__.nhwf(), dm.at(mt, 0, 0), dm.ld(),
-                        &linalg_const<double_complex>::zero(), Up.at(mt, 0, 0), Up.ld());
-        if (ctx__.processing_unit() == device_t::GPU) {
-            Up.copy_to(memory_t::host);
-        }
-    }
-    transform<std::complex<T>, std::complex<T>>(ctx__.spla_context(), spins__(), 1.0, {&hub_wf__}, 0, hub_wf__.num_wf(),
-        Up, 0, 0, 1.0, {&hphi__}, N__, n__);
-}
-
 template class Non_local_operator<double>;
 
 template class D_operator<double>;
 
 template class Q_operator<double>;
 
-template void apply_non_local_d_q<double>(spin_range spins__, int N__, int n__, Beta_projectors<double>& beta__,
-                                          Wave_functions<double>& phi__, D_operator<double>* d_op__,
-                                          Wave_functions<double>* hphi__, Q_operator<double>* q_op__,
-                                          Wave_functions<double>* sphi__);
-
-template void apply_non_local_d_q<double_complex>(spin_range spins__, int N__, int n__, Beta_projectors<double>& beta__,
-                                                  Wave_functions<double>& phi__, D_operator<double>* d_op__,
-                                                  Wave_functions<double>* hphi__, Q_operator<double>* q_op__,
-                                                  Wave_functions<double>* sphi__);
-
-template void apply_S_operator<double>(device_t pu__, spin_range spins__, int N__, int n__,
-                                       Beta_projectors<double>& beta__, Wave_functions<double>& phi__,
-                                       Q_operator<double>* q_op__, Wave_functions<double>& sphi__);
-
-template void apply_S_operator<double_complex>(device_t pu__, spin_range spins__, int N__, int n__,
-                                               Beta_projectors<double>& beta__, Wave_functions<double>& phi__,
-                                               Q_operator<double>* q_op__, Wave_functions<double>& sphi__);
-
-template void apply_U_operator<double>(Simulation_context& ctx__, spin_range spins__, int N__, int n__,
-                                       Wave_functions<double>& hub_wf__, Wave_functions<double>& phi__,
-                                       U_operator<double>& um__, Wave_functions<double>& hphi__);
-
 #if defined(USE_FP32)
 template class Non_local_operator<float>;
 
 template class D_operator<float>;
 
 template class Q_operator<float>;
-
-template void apply_non_local_d_q<float>(spin_range spins__, int N__, int n__, Beta_projectors<float>& beta__,
-                                         Wave_functions<float>& phi__, D_operator<float>* d_op__,
-                                         Wave_functions<float>* hphi__, Q_operator<float>* q_op__,
-                                         Wave_functions<float>* sphi__);
-
-template void apply_non_local_d_q<std::complex<float>>(spin_range spins__, int N__, int n__,
-                                                       Beta_projectors<float>& beta__, Wave_functions<float>& phi__,
-                                                       D_operator<float>* d_op__, Wave_functions<float>* hphi__,
-                                                       Q_operator<float>* q_op__, Wave_functions<float>* sphi__);
-
-template void apply_S_operator<float>(device_t pu__, spin_range spins__, int N__, int n__,
-                                      Beta_projectors<float>& beta__, Wave_functions<float>& phi__,
-                                      Q_operator<float>* q_op__, Wave_functions<float>& sphi__);
-
-template void apply_S_operator<std::complex<float>>(device_t pu__, spin_range spins__, int N__, int n__,
-                                                    Beta_projectors<float>& beta__, Wave_functions<float>& phi__,
-                                                    Q_operator<float>* q_op__, Wave_functions<float>& sphi__);
-
-template void apply_U_operator<float>(Simulation_context& ctx__, spin_range spins__, int N__, int n__,
-                                      Wave_functions<float>& hub_wf__, Wave_functions<float>& phi__,
-                                      U_operator<float>& um__, Wave_functions<float>& hphi__);
 #endif
 } // namespace sirius
