@@ -38,8 +38,10 @@ void K_point<T>::generate_fv_states()
 
     auto const& uc = ctx_.unit_cell();
 
+    auto pcs = env::print_checksum();
+
     auto bs = ctx_.cyclic_block_size();
-    sddk::dmatrix<std::complex<T>> alm_fv(uc.mt_aw_basis_size(), ctx_.num_fv_states(),
+    la::dmatrix<std::complex<T>> alm_fv(uc.mt_aw_basis_size(), ctx_.num_fv_states(),
             ctx_.blacs_grid(), bs, bs);
 
     int atom_begin{0};
@@ -57,6 +59,10 @@ void K_point<T>::generate_fv_states()
 
         /* generate complex conjugated Alm coefficients for a block of atoms */
         auto alm = generate_alm_block<false, T>(ctx_, atom_begin, na, this->alm_coeffs_loc());
+        auto cs = alm.checksum();
+        if (pcs) {
+            utils::print_checksum("alm", cs, RTE_OUT(this->out(0)));
+        }
 
         /* compute F(lm, i) = A(lm, G)^{T} * evec(G, i) for the block of atoms */
         spla::pgemm_ssb(num_mt_aw, ctx_.num_fv_states(), this->gkvec().count(), SPLA_OP_TRANSPOSE, 1.0,
@@ -77,12 +83,12 @@ void K_point<T>::generate_fv_states()
     wf::Wave_functions_mt<T> alm_fv_slab(this->comm(), num_mt_apw_coeffs, wf::num_mag_dims(0),
             wf::num_bands(ctx_.num_fv_states()), sddk::memory_t::host);
 
-    auto& one = sddk::linalg_const<std::complex<T>>::one();
-    auto& zero = sddk::linalg_const<std::complex<T>>::zero();
+    auto& one = la::constant<std::complex<T>>::one();
+    auto& zero = la::constant<std::complex<T>>::zero();
 
     auto layout_in = alm_fv.grid_layout(0, 0, uc.mt_aw_basis_size(), ctx_.num_fv_states());
     auto layout_out = alm_fv_slab.grid_layout_mt(wf::spin_index(0), wf::band_range(0, ctx_.num_fv_states()));
-    costa::transform(layout_in, layout_out, 'N', one, zero, this->comm().mpi_comm());
+    costa::transform(layout_in, layout_out, 'N', one, zero, this->comm().native());
 
     #pragma omp parallel for
     for (int i = 0; i < ctx_.num_fv_states(); i++) {
@@ -106,6 +112,13 @@ void K_point<T>::generate_fv_states()
             }
         }
     }
+    if (pcs) {
+        auto z1 = fv_states_->checksum_pw(sddk::memory_t::host, wf::spin_index(0), wf::band_range(0, ctx_.num_fv_states()));
+        auto z2 = fv_states_->checksum_mt(sddk::memory_t::host, wf::spin_index(0), wf::band_range(0, ctx_.num_fv_states()));
+        utils::print_checksum("fv_states_pw", z1, RTE_OUT(this->out(0)));
+        utils::print_checksum("fv_states_mt", z2, RTE_OUT(this->out(0)));
+
+    }
 }
 
 template void K_point<double>::generate_fv_states();
