@@ -28,7 +28,7 @@
 #include "lapw/matching_coefficients.hpp"
 #include "beta_projectors/beta_projectors.hpp"
 #include "unit_cell/radial_functions_index.hpp"
-#include "SDDK/fft.hpp"
+#include "fft/fft.hpp"
 
 namespace sirius {
 
@@ -49,32 +49,32 @@ class K_point
     Unit_cell const& unit_cell_;
 
     /// Fractional k-point coordinates.
-    vector3d<double> vk_;
+    r3::vector<double> vk_;
 
     /// Weight of k-point.
     double weight_{1.0};
 
     /// Communicator for parallelization inside k-point.
     /** This communicator is used to split G+k vectors and wave-functions. */
-    sddk::Communicator const& comm_;
+    mpi::Communicator const& comm_;
 
     /// List of G-vectors with |G+k| < cutoff.
-    std::shared_ptr<sddk::Gvec> gkvec_;
+    std::shared_ptr<fft::Gvec> gkvec_;
 
-    std::shared_ptr<sddk::Gvec> gkvec_row_;
+    std::shared_ptr<fft::Gvec> gkvec_row_;
 
-    std::shared_ptr<sddk::Gvec> gkvec_col_;
+    std::shared_ptr<fft::Gvec> gkvec_col_;
 
     /// G-vector distribution for the FFT transformation.
-    std::shared_ptr<sddk::Gvec_fft> gkvec_partition_;
+    std::shared_ptr<fft::Gvec_fft> gkvec_partition_;
 
-    std::unique_ptr<spfft_transform_type<T>> spfft_transform_;
+    std::unique_ptr<fft::spfft_transform_type<T>> spfft_transform_;
 
     /// First-variational eigen values
     sddk::mdarray<double, 1> fv_eigen_values_;
 
     /// First-variational eigen vectors, distributed over 2D BLACS grid.
-    sddk::dmatrix<std::complex<T>> fv_eigen_vectors_;
+    la::dmatrix<std::complex<T>> fv_eigen_vectors_;
 
     /// First-variational eigen vectors, distributed in slabs.
     std::unique_ptr<wf::Wave_functions<T>> fv_eigen_vectors_slab_;
@@ -86,7 +86,7 @@ class K_point
     /** Second-variational eigen-vectors are stored as one or two \f$ N_{fv} \times N_{fv} \f$ matrices in
      *  case of non-magnetic or collinear magnetic case or as a single \f$ 2 N_{fv} \times 2 N_{fv} \f$
      *  matrix in case of general non-collinear magnetism. */
-    std::array<sddk::dmatrix<std::complex<T>>, 2> sv_eigen_vectors_;
+    std::array<la::dmatrix<std::complex<T>>, 2> sv_eigen_vectors_;
 
     /// Full-diagonalization eigen vectors.
     sddk::mdarray<std::complex<T>, 2> fd_eigen_vectors_;
@@ -159,7 +159,7 @@ class K_point
     std::vector<std::vector<int>> atom_lo_rows_;
 
     /// Imaginary unit to the power of l.
-    std::vector<double_complex> zil_;
+    std::vector<std::complex<double>> zil_;
 
     /// Mapping between lm and l.
     std::vector<int> l_by_lm_;
@@ -191,10 +191,10 @@ class K_point
     sddk::mdarray<std::complex<T>, 3> p_mtrx_;
 
     /// Communicator between(!!) rows.
-    sddk::Communicator const& comm_row_;
+    mpi::Communicator const& comm_row_;
 
     /// Communicator between(!!) columns.
-    sddk::Communicator const& comm_col_;
+    mpi::Communicator const& comm_col_;
 
     std::array<int, 2> ispn_map_{0, -1};
 
@@ -230,7 +230,7 @@ class K_point
 
   public:
     /// Constructor
-    K_point(Simulation_context& ctx__, vector3d<double> vk__, double weight__)
+    K_point(Simulation_context& ctx__, r3::vector<double> vk__, double weight__)
         : ctx_(ctx__)
         , unit_cell_(ctx_.unit_cell())
         , vk_(vk__)
@@ -244,12 +244,12 @@ class K_point
         , comm_col_(ctx_.blacs_grid().comm_col())
     {
         this->init0();
-        gkvec_ = std::make_shared<sddk::Gvec>(vk_, unit_cell_.reciprocal_lattice_vectors(), ctx_.gk_cutoff(), comm_,
+        gkvec_ = std::make_shared<fft::Gvec>(vk_, unit_cell_.reciprocal_lattice_vectors(), ctx_.gk_cutoff(), comm_,
                                         ctx_.gamma_point());
     }
 
     /// Constructor
-    K_point(Simulation_context& ctx__, std::shared_ptr<sddk::Gvec> gkvec__, double weight__)
+    K_point(Simulation_context& ctx__, std::shared_ptr<fft::Gvec> gkvec__, double weight__)
         : ctx_(ctx__)
         , unit_cell_(ctx_.unit_cell())
         , vk_(gkvec__->vk())
@@ -521,7 +521,7 @@ class K_point
         return *singular_components_;
     }
 
-    inline vector3d<double> vk() const
+    inline auto vk() const
     {
         return vk_;
     }
