@@ -88,7 +88,7 @@ void
 K_point_set::sync_band<float, sync_band_t::occupancy>();
 #endif
 
-void K_point_set::create_k_mesh(vector3d<int> k_grid__, vector3d<int> k_shift__, int use_symmetry__)
+void K_point_set::create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shift__, int use_symmetry__)
 {
     PROFILE("sirius::K_point_set::create_k_mesh");
 
@@ -311,8 +311,8 @@ void K_point_set::find_band_occupancies()
             emax = std::max(emax, this->get<T>(ik)->band_energy(ctx_.num_bands() - 1, ispn));
         }
     }
-    comm().allreduce<double, sddk::mpi_op_t::min>(&emin, 1);
-    comm().allreduce<double, sddk::mpi_op_t::max>(&emax, 1);
+    comm().allreduce<double, mpi::op_t::min>(&emin, 1);
+    comm().allreduce<double, mpi::op_t::max>(&emax, 1);
 
     sddk::splindex<sddk::splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
 
@@ -517,7 +517,7 @@ double K_point_set::entropy_sum() const
 
 void K_point_set::print_info()
 {
-    sddk::pstdout pout(this->comm());
+    mpi::pstdout pout(this->comm());
 
     if (ctx_.comm().rank() == 0) {
         pout << std::endl;
@@ -584,7 +584,7 @@ void K_point_set::load()
     //==     fin["K_point_set"][jk].read("coordinates", vk_in, 3);
     //==     for (int ik = 0; ik < num_kpoints(); ik++)
     //==     {
-    //==         vector3d<double> dvk;
+    //==         r3::vector<double> dvk;
     //==         for (int x = 0; x < 3; x++) dvk[x] = vk_in[x] - kpoints_[ik]->vk(x);
     //==         if (dvk.length() < 1e-12)
     //==         {
@@ -651,7 +651,7 @@ void K_point_set::load()
 //==         fin["kpoints"][jk].read("coordinates", vk_in, 3);
 //==         for (int ik = 0; ik < num_kpoints(); ik++)
 //==         {
-//==             vector3d<double> dvk;
+//==             r3::vector<double> dvk;
 //==             for (int x = 0; x < 3; x++) dvk[x] = vk_in[x] - kpoints_[ik]->vk(x);
 //==             if (dvk.length() < 1e-12)
 //==             {
