@@ -64,7 +64,7 @@ class K_point_set
     K_point_set(K_point_set& src) = delete;
 
     /// Create regular grid of k-points.
-    void create_k_mesh(vector3d<int> k_grid__, vector3d<int> k_shift__, int use_symmetry__);
+    void create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shift__, int use_symmetry__);
 
     bool initialized_{false};
 
@@ -83,7 +83,7 @@ class K_point_set
     }
 
     /// Create a regular mesh of k-points.
-    K_point_set(Simulation_context& ctx__, vector3d<int> k_grid__, vector3d<int> k_shift__, int use_symmetry__)
+    K_point_set(Simulation_context& ctx__, r3::vector<int> k_grid__, r3::vector<int> k_shift__, int use_symmetry__)
         : ctx_(ctx__)
     {
         create_k_mesh(k_grid__, k_shift__, use_symmetry__);
@@ -165,12 +165,12 @@ class K_point_set
             auto ik       = spl_num_kpoints_[ikloc];
             max_num_gkvec = std::max(max_num_gkvec, kpoints_[ik]->num_gkvec());
         }
-        comm().allreduce<int, sddk::mpi_op_t::max>(&max_num_gkvec, 1);
+        comm().allreduce<int, mpi::op_t::max>(&max_num_gkvec, 1);
         return max_num_gkvec;
     }
 
     /// Add k-point to the set.
-    void add_kpoint(vector3d<double> vk__, double weight__)
+    void add_kpoint(r3::vector<double> vk__, double weight__)
     {
         kpoints_.push_back(std::unique_ptr<K_point<double>>(new K_point<double>(ctx_, vk__, weight__)));
 #ifdef USE_FP32
@@ -217,7 +217,7 @@ class K_point_set
     }
 
     /// Find index of k-point.
-    inline int find_kpoint(vector3d<double> vk__)
+    inline int find_kpoint(r3::vector<double> vk__)
     {
         for (int ik = 0; ik < num_kpoints(); ik++) {
             if ((kpoints_[ik]->vk() - vk__).length() < 1e-12) {
@@ -239,7 +239,7 @@ class K_point_set
 
     /// Send G+k vectors of k-point jk to a given rank.
     /** Other ranks receive an empty Gvec placeholder */
-    inline sddk::Gvec get_gkvec(int jk__, int rank__)
+    inline fft::Gvec get_gkvec(int jk__, int rank__)
     {
         /* rank in the k-point communicator */
         int my_rank = comm().rank();
@@ -248,9 +248,9 @@ class K_point_set
         int jrank = spl_num_kpoints().local_rank(jk__);
 
         /* need this to pass communicator */
-        sddk::Gvec gkvec(ctx_.comm_band());
+        fft::Gvec gkvec(ctx_.comm_band());
 
-        sddk::Gvec const* gvptr{nullptr};
+        fft::Gvec const* gvptr{nullptr};
         /* if this rank stores the k-point, then send it */
         if (my_rank == jrank) {
             gvptr = &kpoints_[jk__].get()->gkvec();
