@@ -32,37 +32,37 @@ void Potential::generate_pw_coefs()
 
     double sq_alpha_half = 0.5 * std::pow(speed_of_light, -2);
 
-    int gv_count  = ctx_.gvec_fft().gvec_count_fft();
+    int gv_count  = ctx_.gvec_fft().count();
 
     auto& fft = ctx_.spfft<double>();
 
     /* temporaty output buffer */
-    sddk::mdarray<double_complex, 1> fpw_fft(gv_count);
+    sddk::mdarray<std::complex<double>, 1> fpw_fft(gv_count);
 
     switch (ctx_.valence_relativity()) {
         case relativity_t::iora: {
-            spfft_input<double>(fft, [&](int ir) -> double
-                             {
-                                 double M = 1 - sq_alpha_half * effective_potential().f_rg(ir);
-                                 return ctx_.theta(ir) / std::pow(M, 2);
-                             });
+            fft::spfft_input<double>(fft, [&](int ir) -> double
+            {
+                double M = 1 - sq_alpha_half * effective_potential().f_rg(ir);
+                return ctx_.theta(ir) / std::pow(M, 2);
+            });
             fft.forward(SPFFT_PU_HOST, reinterpret_cast<double*>(&fpw_fft[0]), SPFFT_FULL_SCALING);
             ctx_.gvec_fft().gather_pw_global(&fpw_fft[0], &rm2_inv_pw_[0]);
         }
         case relativity_t::zora: {
-            spfft_input<double>(fft, [&](int ir)
-                             {
-                                 double M = 1 - sq_alpha_half * effective_potential().f_rg(ir);
-                                 return ctx_.theta(ir) / M;
-                             });
+            fft::spfft_input<double>(fft, [&](int ir)
+            {
+                double M = 1 - sq_alpha_half * effective_potential().f_rg(ir);
+                return ctx_.theta(ir) / M;
+            });
             fft.forward(SPFFT_PU_HOST, reinterpret_cast<double*>(&fpw_fft[0]), SPFFT_FULL_SCALING);
             ctx_.gvec_fft().gather_pw_global(&fpw_fft[0], &rm_inv_pw_[0]);
         }
         default: {
-            spfft_input<double>(fft, [&](int ir)
-                             {
-                                 return effective_potential().f_rg(ir) * ctx_.theta(ir);
-                             });
+            fft::spfft_input<double>(fft, [&](int ir)
+            {
+                return effective_potential().f_rg(ir) * ctx_.theta(ir);
+            });
             fft.forward(SPFFT_PU_HOST, reinterpret_cast<double*>(&fpw_fft[0]), SPFFT_FULL_SCALING);
             ctx_.gvec_fft().gather_pw_global(&fpw_fft[0], &veff_pw_[0]);
         }
