@@ -66,7 +66,7 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
         rhomin = std::min(rhomin, d);
         rho.f_rg(ir) = std::max(d, 0.0);
     }
-    sddk::Communicator(ctx_.spfft<double>().communicator()).allreduce<double, sddk::mpi_op_t::min>(&rhomin, 1);
+    mpi::Communicator(ctx_.spfft<double>().communicator()).allreduce<double, mpi::op_t::min>(&rhomin, 1);
     /* even a small negative density is a sign of something bing wrong; don't remove this check */
     if (rhomin < 0.0 && ctx_.comm().rank() == 0) {
         std::stringstream s;
@@ -424,7 +424,7 @@ void Potential::xc_rg_magnetic(Density const& density__)
             /* get the sign between mag and B */
             auto s = utils::sign((rho_up.f_rg(irloc) - rho_dn.f_rg(irloc)) * bxc);
 
-            vector3d<double> m;
+            r3::vector<double> m;
             for (int j = 0; j < ctx_.num_mag_dims(); j++) {
                 m[j] = density__.magnetization(j).f_rg(irloc);
             }
