@@ -170,9 +170,9 @@ class Gaunt_coefficients
      *  \f]
      *  Result is assumed to be complex.
      */
-    inline double_complex sum_L3_gaunt(int lm1, int lm2, double_complex const* v) const
+    inline auto sum_L3_gaunt(int lm1, int lm2, std::complex<double> const* v) const
     {
-        double_complex zsum(0, 0);
+        std::complex<double> zsum(0, 0);
         for (int k = 0; k < (int)gaunt_packed_L3_(lm1, lm2).size(); k++) {
             zsum += gaunt_packed_L3_(lm1, lm2)[k].coef * v[gaunt_packed_L3_(lm1, lm2)[k].lm3];
         }
@@ -201,7 +201,8 @@ class Gaunt_coefficients
         return gaunt_packed_L3_(lm1, lm2);
     }
 
-    inline sddk::mdarray<T, 3> get_full_set_L3() const
+    /// Return the full tensor of Gaunt coefficients <R_{L1}|R_{L3}|R_{L2}> with a (L3, L1, L2) order of indices.
+    inline auto get_full_set_L3() const
     {
         sddk::mdarray<T, 3> gc(lmmax3_, lmmax1_, lmmax2_);
         gc.zero();
