@@ -32,11 +32,11 @@
 #include <string.h>
 #include <vector>
 #include <algorithm>
-#include <geometry3d.hpp>
 #include <constants.hpp>
 
 #include "typedefs.hpp"
 #include "linalg/linalg.hpp"
+#include "linalg/r3.hpp"
 #include "lebedev_grids.hpp"
 #include "specfunc/specfunc.hpp"
 
@@ -49,15 +49,15 @@ wigner_d_matrix(int l, double beta);
 
 template <typename T>
 sddk::mdarray<T, 2>
-rotation_matrix_l(int l, geometry3d::vector3d<double> euler_angles, int proper_rotation);
+rotation_matrix_l(int l, r3::vector<double> euler_angles, int proper_rotation);
 
 template <typename T>
 void
-rotation_matrix(int lmax, geometry3d::vector3d<double> euler_angles, int proper_rotation, sddk::mdarray<T, 2>& rotm);
+rotation_matrix(int lmax, r3::vector<double> euler_angles, int proper_rotation, sddk::mdarray<T, 2>& rotm);
 
 template <typename T>
 std::vector<sddk::mdarray<T, 2>>
-rotation_matrix(int lmax, geometry3d::vector3d<double> euler_angles, int proper_rotation);
+rotation_matrix(int lmax, r3::vector<double> euler_angles, int proper_rotation);
 
 double
 ClebschGordan(const int l, const double j, const double mj, const int spin);
@@ -68,7 +68,7 @@ ClebschGordan(const int l, const double j, const double mj, const int spin);
 
 // mj is normally half integer from -j to j but to avoid computation
 // error it is considered as integer so mj = 2 mj
-double_complex
+std::complex<double>
 calculate_U_sigma_m(const int l, const double j, const int mj, const int mp, const int sigma);
 
 }
@@ -102,10 +102,10 @@ class SHT // TODO: better name
     std::vector<double> w_;
 
     /// Backward transformation from Ylm to spherical coordinates.
-    sddk::mdarray<double_complex, 2> ylm_backward_;
+    sddk::mdarray<std::complex<double>, 2> ylm_backward_;
 
     /// Forward transformation from spherical coordinates to Ylm.
-    sddk::mdarray<double_complex, 2> ylm_forward_;
+    sddk::mdarray<std::complex<double>, 2> ylm_forward_;
 
     /// Backward transformation from Rlm to spherical coordinates.
     sddk::mdarray<double, 2> rlm_backward_;
@@ -162,9 +162,9 @@ class SHT // TODO: better name
             }
         }
 
-        ylm_backward_ = sddk::mdarray<double_complex, 2>(lmmax_, num_points_);
+        ylm_backward_ = sddk::mdarray<std::complex<double>, 2>(lmmax_, num_points_);
 
-        ylm_forward_ = sddk::mdarray<double_complex, 2>(num_points_, lmmax_);
+        ylm_forward_ = sddk::mdarray<std::complex<double>, 2>(num_points_, lmmax_);
 
         rlm_backward_ = sddk::mdarray<double, 2>(lmmax_, num_points_);
 
@@ -177,7 +177,7 @@ class SHT // TODO: better name
                     coord_(1, itp) = y[itp];
                     coord_(2, itp) = z[itp];
 
-                    auto vs = spherical_coordinates(geometry3d::vector3d<double>(x[itp], y[itp], z[itp]));
+                    auto vs = spherical_coordinates(r3::vector<double>(x[itp], y[itp], z[itp]));
 
                     tp_(0, itp) = vs[1];
                     tp_(1, itp) = vs[2];
@@ -210,8 +210,8 @@ class SHT // TODO: better name
         }
 
         if (mesh_type_ == 1) {
-            sddk::linalg(sddk::linalg_t::lapack).geinv(lmmax_, ylm_forward_);
-            sddk::linalg(sddk::linalg_t::lapack).geinv(lmmax_, rlm_forward_);
+            la::wrap(la::lib_t::lapack).geinv(lmmax_, ylm_forward_);
+            la::wrap(la::lib_t::lapack).geinv(lmmax_, rlm_forward_);
         }
 
         switch (pu_) {
@@ -261,7 +261,7 @@ class SHT // TODO: better name
     void forward_transform(T const* ftp, int nr, int lmmax, int ld, T* flm) const;
 
     /// Convert form Rlm to Ylm representation.
-    static void convert(int lmax__, double const* f_rlm__, double_complex* f_ylm__)
+    static void convert(int lmax__, double const* f_rlm__, std::complex<double>* f_ylm__)
     {
         int lm = 0;
         for (int l = 0; l <= lmax__; l++) {
@@ -278,7 +278,7 @@ class SHT // TODO: better name
     }
 
     /// Convert from Ylm to Rlm representation.
-    static void convert(int lmax__, double_complex const* f_ylm__, double* f_rlm__)
+    static void convert(int lmax__, std::complex<double> const* f_ylm__, double* f_rlm__)
     {
         int lm = 0;
         for (int l = 0; l <= lmax__; l++) {
@@ -329,34 +329,6 @@ class SHT // TODO: better name
     //    }
     //}
 
-    /// Transform Cartesian coordinates [x,y,z] to spherical coordinates [r,theta,phi]
-    static geometry3d::vector3d<double> spherical_coordinates(geometry3d::vector3d<double> vc)
-    {
-        geometry3d::vector3d<double> vs;
-
-        const double eps{1e-12};
-
-        vs[0] = vc.length();
-
-        if (vs[0] <= eps) {
-            vs[1] = 0.0;
-            vs[2] = 0.0;
-        } else {
-            vs[1] = std::acos(vc[2] / vs[0]); // theta = cos^{-1}(z/r)
-
-            if (std::abs(vc[0]) > eps || std::abs(vc[1]) > eps) {
-                vs[2] = std::atan2(vc[1], vc[0]); // phi = tan^{-1}(y/x)
-                if (vs[2] < 0.0) {
-                    vs[2] += twopi;
-                }
-            } else {
-                vs[2] = 0.0;
-            }
-        }
-
-        return vs;
-    }
-
     /// Compute element of the transformation matrix from complex to real spherical harmonics.
     /** Real spherical harmonic can be written as a linear combination of complex harmonics:
         \f[
@@ -388,36 +360,36 @@ class SHT // TODO: better name
         Rlm[l_, m_, t_, p_] := Sum[a[m1, m]*SphericalHarmonicY[l, m1, t, p], {m1, -l, l}]
         \endverbatim
      */
-    static inline double_complex ylm_dot_rlm(int l, int m1, int m2)
+    static inline std::complex<double> ylm_dot_rlm(int l, int m1, int m2)
     {
         double const isqrt2 = 1.0 / std::sqrt(2);
 
         assert(l >= 0 && std::abs(m1) <= l && std::abs(m2) <= l);
 
         if (!((m1 == m2) || (m1 == -m2))) {
-            return double_complex(0, 0);
+            return std::complex<double>(0, 0);
         }
 
         if (m1 == 0) {
-            return double_complex(1, 0);
+            return std::complex<double>(1, 0);
         }
 
         if (m1 < 0) {
             if (m2 < 0) {
-                return -double_complex(0, isqrt2);
+                return -std::complex<double>(0, isqrt2);
             } else {
-                return std::pow(-1.0, m2) * double_complex(isqrt2, 0);
+                return std::pow(-1.0, m2) * std::complex<double>(isqrt2, 0);
             }
         } else {
             if (m2 < 0) {
-                return std::pow(-1.0, m1) * double_complex(0, isqrt2);
+                return std::pow(-1.0, m1) * std::complex<double>(0, isqrt2);
             } else {
-                return double_complex(isqrt2, 0);
+                return std::complex<double>(isqrt2, 0);
             }
         }
     }
 
-    static inline double_complex rlm_dot_ylm(int l, int m1, int m2)
+    static inline std::complex<double> rlm_dot_ylm(int l, int m1, int m2)
     {
         return std::conj(ylm_dot_rlm(l, m2, m1));
     }
@@ -503,7 +475,7 @@ class SHT // TODO: better name
      *    \langle Y_{\ell_1 m_1} | R_{\ell_2 m_2} | Y_{\ell_3 m_3} \rangle
      *  \f]
      */
-    static double_complex gaunt_hybrid(int l1, int l2, int l3, int m1, int m2, int m3)
+    static std::complex<double> gaunt_hybrid(int l1, int l2, int l3, int m1, int m2, int m3)
     {
         assert(l1 >= 0);
         assert(l2 >= 0);
@@ -513,7 +485,7 @@ class SHT // TODO: better name
         assert(m3 >= -l3 && m3 <= l3);
 
         if (m2 == 0) {
-            return double_complex(gaunt_yyy(l1, l2, l3, m1, m2, m3), 0.0);
+            return std::complex<double>(gaunt_yyy(l1, l2, l3, m1, m2, m3), 0.0);
         } else {
             return (ylm_dot_rlm(l2, m2, m2) * gaunt_yyy(l1, l2, l3, m1, m2, m3) +
                     ylm_dot_rlm(l2, -m2, m2) * gaunt_yyy(l1, l2, l3, m1, -m2, m3));
@@ -553,57 +525,57 @@ class SHT // TODO: better name
                gsl_sf_coupling_3j(2 * l1, 2 * l2, 2 * l3, 2 * m1, 2 * m2, -2 * m3);
     }
 
-    inline double_complex ylm_backward(int lm, int itp) const
+    inline auto ylm_backward(int lm, int itp) const
     {
         return ylm_backward_(lm, itp);
     }
 
-    inline double rlm_backward(int lm, int itp) const
+    inline auto rlm_backward(int lm, int itp) const
     {
         return rlm_backward_(lm, itp);
     }
 
-    inline double coord(int x, int itp) const
+    inline auto coord(int x, int itp) const
     {
         return coord_(x, itp);
     }
 
-    inline geometry3d::vector3d<double> coord(int idx__) const
+    inline auto coord(int idx__) const
     {
-        return geometry3d::vector3d<double>(coord_(0, idx__), coord_(1, idx__), coord(2, idx__));
+        return r3::vector<double>(coord_(0, idx__), coord_(1, idx__), coord(2, idx__));
     }
 
-    inline double theta(int idx__) const
+    inline auto theta(int idx__) const
     {
         return tp_(0, idx__);
     }
 
-    inline double phi(int idx__) const
+    inline auto phi(int idx__) const
     {
         return tp_(1, idx__);
     }
 
-    inline double weight(int idx__) const
+    inline auto weight(int idx__) const
     {
         return w_[idx__];
     }
 
-    inline int num_points() const
+    inline auto num_points() const
     {
         return num_points_;
     }
 
-    inline int lmax() const
+    inline auto lmax() const
     {
         return lmax_;
     }
 
-    inline int lmmax() const
+    inline auto lmmax() const
     {
         return lmmax_;
     }
-
 };
+
 } // namespace sirius
 
 #endif // __SHT_HPP__
