@@ -34,9 +34,6 @@
 #include "gpu/cusolver.hpp"
 #endif
 #include "linalg/linalg_spla.hpp"
-#if defined(SIRIUS_ELPA)
-#include "linalg/elpa.hpp"
-#endif
 #include "utils/cmd_args.hpp"
 #include "utils/json.hpp"
 #include "utils/profiler.hpp"
@@ -53,17 +50,13 @@ using json = nlohmann::json;
 #include "hdf5_tree.hpp"
 #include "band/band.hpp"
 #include "dft/dft_ground_state.hpp"
+#include "linalg/eigenproblem.hpp"
 #include "sirius_version.hpp"
 
 #if defined(__PLASMA)
 extern "C" void plasma_init(int num_cores);
 #endif
 
-#if defined(__LIBSCI_ACC)
-extern "C" void libsci_acc_init();
-extern "C" void libsci_acc_finalize();
-#endif
-
 /// Namespace of the SIRIUS library.
 namespace sirius {
 
@@ -103,13 +96,13 @@ inline void initialize(bool call_mpi_init__ = true)
     energy_acc() = -utils::power::device_energy();
 #endif
     if (call_mpi_init__) {
-        sddk::Communicator::initialize(MPI_THREAD_MULTIPLE);
+        mpi::Communicator::initialize(MPI_THREAD_MULTIPLE);
     }
 #if defined(__APEX)
     apex::init("sirius", Communicator::world().rank(), Communicator::world().size());
 #endif
 
-    if (sddk::Communicator::world().rank() == 0) {
+    if (mpi::Communicator::world().rank() == 0) {
         std::printf("SIRIUS %i.%i.%i, git hash: %s\n", sirius::major_version(), sirius::minor_version(),
                sirius::revision(), sirius::git_hash().c_str());
 #if !defined(NDEBUG)
@@ -117,9 +110,9 @@ inline void initialize(bool call_mpi_init__ = true)
 #endif
     }
     /* get number of ranks per node during the global call to sirius::initialize() */
-    sddk::num_ranks_per_node();
+    mpi::num_ranks_per_node();
     if (acc::num_devices() > 0) {
-        int devid = sddk::get_device_id(acc::num_devices());
+        int devid = mpi::get_device_id(acc::num_devices());
         acc::set_device_id(devid);
         /* create extensive amount of streams */
         /* some parts of the code rely on the number of streams not related to the
@@ -141,13 +134,8 @@ inline void initialize(bool call_mpi_init__ = true)
 #if defined(__PLASMA)
     plasma_init(omp_get_max_threads());
 #endif
-#if defined(__LIBSCI_ACC)
-    libsci_acc_init();
-#endif
 #if defined(SIRIUS_ELPA)
-    if (elpa_init(20170403) != ELPA_OK) {
-        TERMINATE("ELPA API version not supported");
-    }
+    la::Eigensolver_elpa::initialize();
 #endif
     /* for the fortran interface to blas/lapack */
     assert(sizeof(int) == 4);
@@ -166,11 +154,8 @@ inline void finalize(bool call_mpi_fin__ = true, bool reset_device__ = true, boo
 #if defined(SIRIUS_MAGMA)
     magma::finalize();
 #endif
-#if defined(__LIBSCI_ACC)
-    libsci_acc_finalize();
-#endif
 
-    // must be called before device is reset
+    /* must be called before device is reset */
     splablas::reset_handle();
 
     sddk::get_memory_pool(sddk::memory_t::host).clear();
@@ -191,20 +176,19 @@ inline void finalize(bool call_mpi_fin__ = true, bool reset_device__ = true, boo
         }
     }
 
-    //utils::stop_global_timer();
 #if defined(__APEX)
     apex::finalize();
 #endif
 #if defined(SIRIUS_USE_POWER_COUNTER)
     double e = energy() + utils::power::energy();
     double e_acc = energy_acc() + utils::power::device_energy();
-    if (sddk::Communicator::world().rank() == 0) {
+    if (mpi::Communicator::world().rank() == 0) {
         printf("=== Energy consumption (root MPI rank) ===\n");
         printf("energy     : %9.2f Joules\n", e);
         printf("energy_acc : %9.2f Joules\n", e_acc);
     }
-    sddk::Communicator::world().allreduce(&e, 1);
-    sddk::Communicator::world().allreduce(&e_acc, 1);
+    mpi::Communicator::world().allreduce(&e, 1);
+    mpi::Communicator::world().allreduce(&e_acc, 1);
     int nn = utils::power::num_nodes();
     if (Communicator::world().rank() == 0 && nn > 0) {
         printf("=== Energy consumption (all nodes) ===\n");
@@ -212,13 +196,12 @@ inline void finalize(bool call_mpi_fin__ = true, bool reset_device__ = true, boo
         printf("energy_acc : %9.2f Joules\n", e_acc * nn / Communicator::world().size());
     }
 #endif
-    auto rank = sddk::Communicator::world().rank();
+    auto rank = mpi::Communicator::world().rank();
     if (call_mpi_fin__) {
-        sddk::Communicator::finalize();
+        mpi::Communicator::finalize();
     }
 #if defined(SIRIUS_ELPA)
-    int ierr;
-    elpa_uninit(&ierr);
+    la::Eigensolver_elpa::finalize();
 #endif
 
     is_initialized() = false;
