@@ -26,11 +26,12 @@
 #define __SYMMETRIZE_HPP__
 
 #include "crystal_symmetry.hpp"
-#include "SDDK/gvec.hpp"
+#include "fft/gvec.hpp"
 #include "SDDK/omp.hpp"
 #include "typedefs.hpp"
 #include "sht/sht.hpp"
 #include "utils/profiler.hpp"
+#include "utils/rte.hpp"
 
 namespace sirius {
 
@@ -94,24 +95,24 @@ namespace sirius {
     \f]
  */
 inline void
-symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__,
-           sddk::mdarray<double_complex, 3> const& sym_phase_factors__, double_complex* f_pw__, double_complex* x_pw__,
-           double_complex* y_pw__, double_complex* z_pw__)
+symmetrize(Crystal_symmetry const& sym__, fft::Gvec_shells const& gvec_shells__,
+           sddk::mdarray<std::complex<double>, 3> const& sym_phase_factors__, std::complex<double>* f_pw__, std::complex<double>* x_pw__,
+           std::complex<double>* y_pw__, std::complex<double>* z_pw__)
 {
     PROFILE("sirius::symmetrize|fpw");
 
-    auto f_pw = f_pw__ ? gvec_shells__.remap_forward(f_pw__) : std::vector<double_complex>();
-    auto x_pw = x_pw__ ? gvec_shells__.remap_forward(x_pw__) : std::vector<double_complex>();
-    auto y_pw = y_pw__ ? gvec_shells__.remap_forward(y_pw__) : std::vector<double_complex>();
-    auto z_pw = z_pw__ ? gvec_shells__.remap_forward(z_pw__) : std::vector<double_complex>();
+    auto f_pw = f_pw__ ? gvec_shells__.remap_forward(f_pw__) : std::vector<std::complex<double>>();
+    auto x_pw = x_pw__ ? gvec_shells__.remap_forward(x_pw__) : std::vector<std::complex<double>>();
+    auto y_pw = y_pw__ ? gvec_shells__.remap_forward(y_pw__) : std::vector<std::complex<double>>();
+    auto z_pw = z_pw__ ? gvec_shells__.remap_forward(z_pw__) : std::vector<std::complex<double>>();
 
     /* local number of G-vectors in a distribution with complete G-vector shells */
     int ngv = gvec_shells__.gvec_count_remapped();
 
-    auto sym_f_pw = f_pw__ ? std::vector<double_complex>(ngv, 0) : std::vector<double_complex>();
-    auto sym_x_pw = x_pw__ ? std::vector<double_complex>(ngv, 0) : std::vector<double_complex>();
-    auto sym_y_pw = y_pw__ ? std::vector<double_complex>(ngv, 0) : std::vector<double_complex>();
-    auto sym_z_pw = z_pw__ ? std::vector<double_complex>(ngv, 0) : std::vector<double_complex>();
+    auto sym_f_pw = f_pw__ ? std::vector<std::complex<double>>(ngv, 0) : std::vector<std::complex<double>>();
+    auto sym_x_pw = x_pw__ ? std::vector<std::complex<double>>(ngv, 0) : std::vector<std::complex<double>>();
+    auto sym_y_pw = y_pw__ ? std::vector<std::complex<double>>(ngv, 0) : std::vector<std::complex<double>>();
+    auto sym_z_pw = z_pw__ ? std::vector<std::complex<double>>(ngv, 0) : std::vector<std::complex<double>>();
 
     bool is_non_collin = ((x_pw__ != nullptr) && (y_pw__ != nullptr) && (z_pw__ != nullptr));
 
@@ -119,7 +120,7 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
 
     double norm = 1 / double(sym__.size());
 
-    auto phase_factor = [&](int isym, vector3d<int> G) {
+    auto phase_factor = [&](int isym, r3::vector<int> G) {
         return sym_phase_factors__(0, G[0], isym) * sym_phase_factors__(1, G[1], isym) *
                sym_phase_factors__(2, G[2], isym);
     };
@@ -140,21 +141,26 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
 
 #if !defined(NDEBUG)
             if (igsh != gvec_shells__.gvec().shell(G)) {
-                throw std::runtime_error("wrong index of G-shell");
+                std::stringstream s;
+                s << "wrong index of G-shell" << std::endl
+                  << "  G-vector : " << G << std::endl
+                  << "  igsh in the remapped set : " << igsh << std::endl
+                  << "  igsh in the original set : " << gvec_shells__.gvec().shell(G);
+                RTE_THROW(s);
             }
 #endif
             /* each thread is working on full shell of G-vectors */
             if (igsh % nt == tid && !is_done[igloc]) {
 
-                double_complex symf(0, 0);
-                double_complex symx(0, 0);
-                double_complex symy(0, 0);
-                double_complex symz(0, 0);
+                std::complex<double> symf(0, 0);
+                std::complex<double> symx(0, 0);
+                std::complex<double> symy(0, 0);
+                std::complex<double> symz(0, 0);
 
                 /* find the symmetrized PW coefficient */
 
                 for (int i = 0; i < sym__.size(); i++) {
-                    auto G1 = dot(G, sym__[i].spg_op.R);
+                    auto G1 = r3::dot(G, sym__[i].spg_op.R);
 
                     auto S = sym__[i].spin_rotation;
 
@@ -163,15 +169,24 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
                     /* local index of a rotated G-vector */
                     int ig1 = gvec_shells__.index_by_gvec(G1);
 
+                    /* check the reduced G-vector */
                     if (ig1 == -1) {
                         G1 = G1 * (-1);
 #if !defined(NDEBUG)
                         if (igsh != gvec_shells__.gvec().shell(G1)) {
-                            throw std::runtime_error("wrong index of G-shell");
+                            std::stringstream s;
+                            s << "wrong index of G-shell" << std::endl
+                              << "  index of G-shell : " << igsh << std::endl
+                              << "  symmetry operation : " << sym__[i].spg_op.R << std::endl
+                              << "  G-vector : " << G << std::endl
+                              << "  rotated G-vector : " << G1 << std::endl
+                              << "  G-vector index : " << gvec_shells__.index_by_gvec(G1) << std::endl
+                              << "  index of rotated G-vector shell : " << gvec_shells__.gvec().shell(G1);
+                            RTE_THROW(s);
                         }
 #endif
                         ig1 = gvec_shells__.index_by_gvec(G1);
-                        assert(ig1 >= 0 && ig1 < ngv);
+                        RTE_ASSERT(ig1 >= 0 && ig1 < ngv);
                         if (f_pw__) {
                             symf += std::conj(f_pw[ig1]) * phase;
                         }
@@ -179,7 +194,7 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
                             symz += std::conj(z_pw[ig1]) * phase * S(2, 2);
                         }
                         if (is_non_collin) {
-                            auto v = dot(S, vector3d<double_complex>({x_pw[ig1], y_pw[ig1], z_pw[ig1]}));
+                            auto v = r3::dot(S, r3::vector<std::complex<double>>({x_pw[ig1], y_pw[ig1], z_pw[ig1]}));
                             symx += std::conj(v[0]) * phase;
                             symy += std::conj(v[1]) * phase;
                             symz += std::conj(v[2]) * phase;
@@ -187,10 +202,18 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
                     } else {
 #if !defined(NDEBUG)
                         if (igsh != gvec_shells__.gvec().shell(G1)) {
-                            throw std::runtime_error("wrong index of G-shell");
+                            std::stringstream s;
+                            s << "wrong index of G-shell" << std::endl
+                              << "  index of G-shell : " << igsh << std::endl
+                              << "  symmetry operation : " << sym__[i].spg_op.R << std::endl
+                              << "  G-vector : " << G << std::endl
+                              << "  rotated G-vector : " << G1 << std::endl
+                              << "  G-vector index : " << gvec_shells__.index_by_gvec(G1) << std::endl
+                              << "  index of rotated G-vector shell : " << gvec_shells__.gvec().shell(G1);
+                            RTE_THROW(s);
                         }
 #endif
-                        assert(ig1 >= 0 && ig1 < ngv);
+                        RTE_ASSERT(ig1 >= 0 && ig1 < ngv);
                         if (f_pw__) {
                             symf += f_pw[ig1] * phase;
                         }
@@ -198,7 +221,7 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
                             symz += z_pw[ig1] * phase * S(2, 2);
                         }
                         if (is_non_collin) {
-                            auto v = dot(S, vector3d<double_complex>({x_pw[ig1], y_pw[ig1], z_pw[ig1]}));
+                            auto v = r3::dot(S, r3::vector<std::complex<double>>({x_pw[ig1], y_pw[ig1], z_pw[ig1]}));
                             symx += v[0] * phase;
                             symy += v[1] * phase;
                             symz += v[2] * phase;
@@ -216,14 +239,14 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
                 for (int isym = 0; isym < sym__.size(); isym++) {
                     auto S = sym__[isym].spin_rotation;
 
-                    auto G1 = dot(sym__[isym].spg_op.invRT, G);
+                    auto G1 = r3::dot(sym__[isym].spg_op.invRT, G);
                     /* index of a rotated G-vector */
                     int ig1 = gvec_shells__.index_by_gvec(G1);
 
                     if (ig1 != -1) {
-                        assert(ig1 >= 0 && ig1 < ngv);
+                        RTE_ASSERT(ig1 >= 0 && ig1 < ngv);
                         auto phase = std::conj(phase_factor(isym, G1));
-                        double_complex symf1, symx1, symy1, symz1;
+                        std::complex<double> symf1, symx1, symy1, symz1;
                         if (f_pw__) {
                             symf1 = symf * phase;
                         }
@@ -231,7 +254,7 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
                             symz1 = symz * phase * S(2, 2);
                         }
                         if (is_non_collin) {
-                            auto v = dot(S, vector3d<double_complex>({symx, symy, symz}));
+                            auto v = r3::dot(S, r3::vector<std::complex<double>>({symx, symy, symz}));
                             symx1  = v[0] * phase;
                             symy1  = v[1] * phase;
                             symz1  = v[2] * phase;
@@ -293,10 +316,10 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
         auto G = gvec_shells__.gvec_remapped(igloc);
         for (int isym = 0; isym < sym__.size(); isym++) {
             auto S      = sym__[isym].spin_rotation;
-            auto gv_rot = dot(sym__[isym].spg_op.invRT, G);
+            auto gv_rot = r3::dot(sym__[isym].spg_op.invRT, G);
             /* index of a rotated G-vector */
             int ig_rot           = gvec_shells__.index_by_gvec(gv_rot);
-            double_complex phase = std::conj(phase_factor(isym, gv_rot));
+            std::complex<double> phase = std::conj(phase_factor(isym, gv_rot));
 
             if (f_pw__ && ig_rot != -1) {
                 if (utils::abs_diff(sym_f_pw[ig_rot], sym_f_pw[igloc] * phase) > eps) {
@@ -333,7 +356,7 @@ symmetrize(Crystal_symmetry const& sym__, sddk::Gvec_shells const& gvec_shells__
 }
 
 inline void
-symmetrize_function(Crystal_symmetry const& sym__, sddk::Communicator const& comm__, sddk::mdarray<double, 3>& frlm__)
+symmetrize_function(Crystal_symmetry const& sym__, mpi::Communicator const& comm__, sddk::mdarray<double, 3>& frlm__)
 {
     PROFILE("sirius::symmetrize_function|flm");
 
@@ -363,9 +386,9 @@ symmetrize_function(Crystal_symmetry const& sym__, sddk::Communicator const& com
         for (int ialoc = 0; ialoc < spl_atoms.local_size(); ialoc++) {
             int ia = spl_atoms[ialoc];
             int ja = sym__[i].spg_op.inv_sym_atom[ia];
-            sddk::linalg(sddk::linalg_t::blas)
+            la::wrap(la::lib_t::blas)
                 .gemm('N', 'N', lmmax, nrmax, lmmax, &alpha, rotm.at(sddk::memory_t::host), rotm.ld(),
-                      frlm__.at(sddk::memory_t::host, 0, 0, ja), frlm__.ld(), &sddk::linalg_const<double>::one(),
+                      frlm__.at(sddk::memory_t::host, 0, 0, ja), frlm__.ld(), &la::constant<double>::one(),
                       fsym.at(sddk::memory_t::host, 0, 0, ialoc), fsym.ld());
         }
     }
@@ -375,7 +398,7 @@ symmetrize_function(Crystal_symmetry const& sym__, sddk::Communicator const& com
 }
 
 inline void
-symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator const& comm__, sddk::mdarray<double, 3>& vz_rlm__)
+symmetrize_vector_function(Crystal_symmetry const& sym__, mpi::Communicator const& comm__, sddk::mdarray<double, 3>& vz_rlm__)
 {
     PROFILE("sirius::symmetrize_function|vzlm");
 
@@ -408,9 +431,9 @@ symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator con
             int ia   = spl_atoms[ialoc];
             int ja   = sym__[i].spg_op.inv_sym_atom[ia];
             double a = alpha * S(2, 2);
-            sddk::linalg(sddk::linalg_t::blas)
+            la::wrap(la::lib_t::blas)
                 .gemm('N', 'N', lmmax, nrmax, lmmax, &a, rotm.at(sddk::memory_t::host), rotm.ld(),
-                      vz_rlm__.at(sddk::memory_t::host, 0, 0, ja), vz_rlm__.ld(), &sddk::linalg_const<double>::one(),
+                      vz_rlm__.at(sddk::memory_t::host, 0, 0, ja), vz_rlm__.ld(), &la::constant<double>::one(),
                       fsym.at(sddk::memory_t::host, 0, 0, ialoc), fsym.ld());
         }
     }
@@ -421,7 +444,7 @@ symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator con
 }
 
 inline void
-symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator const& comm__, sddk::mdarray<double, 3>& vx_rlm__,
+symmetrize_vector_function(Crystal_symmetry const& sym__, mpi::Communicator const& comm__, sddk::mdarray<double, 3>& vx_rlm__,
                            sddk::mdarray<double, 3>& vy_rlm__, sddk::mdarray<double, 3>& vz_rlm__)
 {
     PROFILE("sirius::symmetrize_function|vlm");
@@ -455,9 +478,9 @@ symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator con
             int ia = spl_atoms[ialoc];
             int ja = sym__[i].spg_op.inv_sym_atom[ia];
             for (int k : {0, 1, 2}) {
-                sddk::linalg(sddk::linalg_t::blas)
+                la::wrap(la::lib_t::blas)
                     .gemm('N', 'N', lmmax, nrmax, lmmax, &alpha, rotm.at(sddk::memory_t::host), rotm.ld(),
-                          vrlm[k]->at(sddk::memory_t::host, 0, 0, ja), vrlm[k]->ld(), &sddk::linalg_const<double>::zero(),
+                          vrlm[k]->at(sddk::memory_t::host, 0, 0, ja), vrlm[k]->ld(), &la::constant<double>::zero(),
                           vtmp.at(sddk::memory_t::host, 0, 0, k), vtmp.ld());
             }
             #pragma omp parallel
@@ -486,18 +509,18 @@ symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator con
 
 /* we compute \f[ Oc = (R_l\cross R_s) . O . (R_l\cross R_s)^\dagger \f] */
 
-// inline void apply_symmetry(const mdarray<double_complex, 3> &dm_,
+// inline void apply_symmetry(const mdarray<std::complex<double>, 3> &dm_,
 //                           const mdarray<double, 2> &rot,
-//                           const mdarray<double_complex, 2> &spin_rot_su2,
+//                           const mdarray<std::complex<double>, 2> &spin_rot_su2,
 //                           const int num_mag_dims_,
 //                           const int l,
-//                           mdarray<double_complex, 3> &res_)
+//                           mdarray<std::complex<double>, 3> &res_)
 //{
 //    res_.zero();
 //    for (int lm1 = 0; lm1 <= 2 * l + 1; lm1++) {
 //        for (int lm2 = 0; lm2 <= 2 * l + 1; lm2++) {
 //            res_.zero();
-//            double_complex dm_rot_spatial[3];
+//            std::complex<double> dm_rot_spatial[3];
 //
 //            for (int j = 0; j < num_mag_dims_; j++) {
 //                // this is a matrix-matrix multiplication P A P ^-1
@@ -515,7 +538,7 @@ symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator con
 //                }
 //            else {
 //                // full non collinear magnetism
-//                double_complex spin_dm[2][2] = {
+//                std::complex<double> spin_dm[2][2] = {
 //                    {dm_rot_spatial[0], dm_rot_spatial[2]},
 //                    {std::conj(dm_rot_spatial[2]), dm_rot_spatial[1]}};
 //
@@ -546,9 +569,9 @@ symmetrize_vector_function(Crystal_symmetry const& sym__, sddk::Communicator con
  *  radial integrals over the total angular momentum
  */
 inline void
-symmetrize(const sddk::mdarray<double_complex, 4>& ns_, basis_functions_index const& indexb, const int ia, const int ja,
-           const int ndm, sddk::mdarray<double, 2> const& rotm, sddk::mdarray<double_complex, 2> const& spin_rot_su2,
-           sddk::mdarray<double_complex, 4>& dm_, const bool hubbard_)
+symmetrize(const sddk::mdarray<std::complex<double>, 4>& ns_, basis_functions_index const& indexb, const int ia, const int ja,
+           const int ndm, sddk::mdarray<double, 2> const& rotm, sddk::mdarray<std::complex<double>, 2> const& spin_rot_su2,
+           sddk::mdarray<std::complex<double>, 4>& dm_, const bool hubbard_)
 {
     for (int xi1 = 0; xi1 < indexb.size(); xi1++) {
         int l1  = indexb[xi1].l;
@@ -563,7 +586,7 @@ symmetrize(const sddk::mdarray<double_complex, 4>& ns_, basis_functions_index co
             int l2                                       = indexb[xi2].l;
             int lm2                                      = indexb[xi2].lm;
             int o2                                       = indexb[xi2].order;
-            std::array<double_complex, 3> dm_rot_spatial = {0, 0, 0};
+            std::array<std::complex<double>, 3> dm_rot_spatial = {0, 0, 0};
 
             //} the hubbard treatment when spin orbit coupling is present is
             // foundamentally wrong since we consider the full hubbard
@@ -594,7 +617,7 @@ symmetrize(const sddk::mdarray<double_complex, 4>& ns_, basis_functions_index co
             if (ndm == 1) {
                 dm_(xi1, xi2, 0, ja) += dm_rot_spatial[0];
             } else {
-                double_complex spin_dm[2][2] = {{dm_rot_spatial[0], dm_rot_spatial[2]},
+                std::complex<double> spin_dm[2][2] = {{dm_rot_spatial[0], dm_rot_spatial[2]},
                                                 {std::conj(dm_rot_spatial[2]), dm_rot_spatial[1]}};
 
                 /* spin blocks of density matrix are: uu, dd, ud
@@ -616,7 +639,7 @@ symmetrize(const sddk::mdarray<double_complex, 4>& ns_, basis_functions_index co
 }
 
 inline void
-symmetrize(std::function<sddk::mdarray<double_complex, 3>&(int ia__)> dm__, int num_mag_comp__,
+symmetrize(std::function<sddk::mdarray<std::complex<double>, 3>&(int ia__)> dm__, int num_mag_comp__,
            Crystal_symmetry const& sym__,
            std::function<sirius::experimental::basis_functions_index const*(int)> indexb__)
 {
@@ -625,11 +648,11 @@ symmetrize(std::function<sddk::mdarray<double_complex, 3>&(int ia__)> dm__, int
         return;
     }
 
-    std::vector<sddk::mdarray<double_complex, 3>> dmsym(sym__.num_atoms());
+    std::vector<sddk::mdarray<std::complex<double>, 3>> dmsym(sym__.num_atoms());
     for (int ia = 0; ia < sym__.num_atoms(); ia++) {
         int iat = sym__.atom_type(ia);
         if (indexb__(iat)) {
-            dmsym[ia] = sddk::mdarray<double_complex, 3>(indexb__(iat)->size(), indexb__(iat)->size(), 4);
+            dmsym[ia] = sddk::mdarray<std::complex<double>, 3>(indexb__(iat)->size(), indexb__(iat)->size(), 4);
             dmsym[ia].zero();
         }
     }
@@ -661,7 +684,7 @@ symmetrize(std::function<sddk::mdarray<double_complex, 3>&(int ia__)> dm__, int
             auto& indexr = indexb.indexr();
 
             int mmax = 2 * indexb.indexr().lmax() + 1;
-            sddk::mdarray<double_complex, 3> dm_ia(mmax, mmax, num_mag_comp__);
+            sddk::mdarray<std::complex<double>, 3> dm_ia(mmax, mmax, num_mag_comp__);
 
             /* loop over radial functions */
             for (int idxrf1 = 0; idxrf1 < indexr.size(); idxrf1++) {
@@ -705,7 +728,7 @@ symmetrize(std::function<sddk::mdarray<double_complex, 3>&(int ia__)> dm__, int
 
                             for (int m1 = 0; m1 < ss1; m1++) {
                                 for (int m2 = 0; m2 < ss2; m2++) {
-                                    double_complex dm[2][2] = {{dm_ia(m1, m2, 0), 0}, {0, dm_ia(m1, m2, 1)}};
+                                    std::complex<double> dm[2][2] = {{dm_ia(m1, m2, 0), 0}, {0, dm_ia(m1, m2, 1)}};
                                     if (num_mag_comp__ == 3) {
                                         dm[0][1] = dm_ia(m1, m2, 2);
                                         dm[1][0] = std::conj(dm[0][1]);
