@@ -29,12 +29,12 @@
 #include <iostream>
 #include <vector>
 #include <cmath>
-#include "SDDK/dmatrix.hpp"
-#include "SDDK/geometry3d.hpp"
 #include "SDDK/wave_functions.hpp"
-#include "utils/profiler.hpp"
 #include "linalg/linalg.hpp"
+#include "linalg/dmatrix.hpp"
+#include "linalg/r3.hpp"
 #include "utils/cmd_args.hpp"
+#include "utils/profiler.hpp"
 #include "context/simulation_context.hpp"
 
 namespace sirius {
@@ -108,12 +108,12 @@ class Measurement: public std::vector<double>
 
 template <typename T>
 inline auto
-random_symmetric(int N__, int bs__, sddk::BLACS_grid const& blacs_grid__)
+random_symmetric(int N__, int bs__, la::BLACS_grid const& blacs_grid__)
 {
     PROFILE("random_symmetric");
 
-    sddk::dmatrix<T> A(N__, N__, blacs_grid__, bs__, bs__);
-    sddk::dmatrix<T> B(N__, N__, blacs_grid__, bs__, bs__);
+    la::dmatrix<T> A(N__, N__, blacs_grid__, bs__, bs__);
+    la::dmatrix<T> B(N__, N__, blacs_grid__, bs__, bs__);
     for (int j = 0; j < A.num_cols_local(); j++) {
         for (int i = 0; i < A.num_rows_local(); i++) {
             A(i, j) = utils::random<T>();
@@ -121,7 +121,7 @@ random_symmetric(int N__, int bs__, sddk::BLACS_grid const& blacs_grid__)
     }
 
 #ifdef SIRIUS_SCALAPACK
-    sddk::linalg(sddk::linalg_t::scalapack).tranc(N__, N__, A, 0, 0, B, 0, 0);
+    la::wrap(la::lib_t::scalapack).tranc(N__, N__, A, 0, 0, B, 0, 0);
 #else
     for (int i = 0; i < N__; i++) {
         for (int j = 0; j < N__; j++) {
@@ -145,13 +145,13 @@ random_symmetric(int N__, int bs__, sddk::BLACS_grid const& blacs_grid__)
 
 template <typename T>
 inline auto
-random_positive_definite(int N__, int bs__, sddk::BLACS_grid const& blacs_grid__)
+random_positive_definite(int N__, int bs__, la::BLACS_grid const& blacs_grid__)
 {
     PROFILE("random_positive_definite");
 
     double p = 1.0 / N__;
-    sddk::dmatrix<T> A(N__, N__, blacs_grid__, bs__, bs__);
-    sddk::dmatrix<T> B(N__, N__, blacs_grid__, bs__, bs__);
+    la::dmatrix<T> A(N__, N__, blacs_grid__, bs__, bs__);
+    la::dmatrix<T> B(N__, N__, blacs_grid__, bs__, bs__);
     for (int j = 0; j < A.num_cols_local(); j++) {
         for (int i = 0; i < A.num_rows_local(); i++) {
             A(i, j) = p * utils::random<T>();
@@ -159,11 +159,11 @@ random_positive_definite(int N__, int bs__, sddk::BLACS_grid const& blacs_grid__
     }
 
 #ifdef SIRIUS_SCALAPACK
-    sddk::linalg(sddk::linalg_t::scalapack).gemm('C', 'N', N__, N__, N__, &sddk::linalg_const<T>::one(), A, 0, 0, A, 0, 0,
-        &sddk::linalg_const<T>::zero(), B, 0, 0);
+    la::wrap(la::lib_t::scalapack).gemm('C', 'N', N__, N__, N__, &la::constant<T>::one(), A, 0, 0, A, 0, 0,
+        &la::constant<T>::zero(), B, 0, 0);
 #else
-    sddk::linalg(sddk::linalg_t::blas).gemm('C', 'N', N__, N__, N__, &sddk::linalg_const<T>::one(), &A(0, 0), A.ld(),
-            &A(0, 0), A.ld(), &sddk::linalg_const<T>::zero(), &B(0, 0), B.ld());
+    la::wrap(la::lib_t::blas).gemm('C', 'N', N__, N__, N__, &la::constant<T>::one(), &A(0, 0), A.ld(),
+            &A(0, 0), A.ld(), &la::constant<T>::zero(), &B(0, 0), B.ld());
 #endif
 
     for (int i = 0; i < N__; i++) {
@@ -174,8 +174,8 @@ random_positive_definite(int N__, int bs__, sddk::BLACS_grid const& blacs_grid__
 }
 
 inline auto
-create_simulation_context(nlohmann::json const& conf__, geometry3d::matrix3d<double> L__, int num_atoms__,
-std::vector<geometry3d::vector3d<double>> coord__, bool add_vloc__, bool add_dion__)
+create_simulation_context(nlohmann::json const& conf__, r3::matrix<double> L__, int num_atoms__,
+std::vector<r3::vector<double>> coord__, bool add_vloc__, bool add_dion__)
 {
     auto ctx = std::make_unique<sirius::Simulation_context>(conf__);
 
@@ -261,7 +261,7 @@ randomize(wf::Wave_functions<T>& wf__)
         for (int s = 0; s < wf__.num_sc().get(); s++) {
             auto ptr = wf__.at(sddk::memory_t::host, 0, wf::spin_index(s), wf::band_index(i));
             for (int j = 0; j < wf__.ld(); j++) {
-                ptr[j] = utils::random<double_complex>();
+                ptr[j] = utils::random<std::complex<double>>();
             }
         }
     }
