@@ -30,8 +30,6 @@
 #include "function3d/spheric_function.hpp"
 #include "utils/profiler.hpp"
 
-using namespace geometry3d;
-
 namespace sirius {
 
 /// Data and methods specific to the actual atom in the unit cell.
@@ -45,10 +43,10 @@ class Atom
     std::shared_ptr<Atom_symmetry_class> symmetry_class_;
 
     /// Position in fractional coordinates.
-    vector3d<double> position_;
+    r3::vector<double> position_;
 
     /// Vector field associated with the current site.
-    vector3d<double> vector_field_;
+    r3::vector<double> vector_field_;
 
     /// Muffin-tin potential.
     sddk::mdarray<double, 2> veff_;
@@ -69,10 +67,10 @@ class Atom
     int offset_lo_{-1}; // TODO: better name for this
 
     /// Unsymmetrized (sampled over IBZ) occupation matrix of the L(S)DA+U method.
-    sddk::mdarray<double_complex, 4> occupation_matrix_;
+    sddk::mdarray<std::complex<double>, 4> occupation_matrix_;
 
     /// U,J correction matrix of the L(S)DA+U method
-    sddk::mdarray<double_complex, 4> uj_correction_matrix_;
+    sddk::mdarray<std::complex<double>, 4> uj_correction_matrix_;
 
     /// True if UJ correction is applied for the current atom.
     bool apply_uj_correction_{false};
@@ -92,7 +90,7 @@ class Atom
 
   public:
     /// Constructor.
-    Atom(Atom_type const& type__, vector3d<double> position__, vector3d<double> vector_field__)
+    Atom(Atom_type const& type__, r3::vector<double> position__, r3::vector<double> vector_field__)
         : type_(type__)
         , position_(position__)
         , vector_field_(vector_field__)
@@ -118,9 +116,9 @@ class Atom
                 b_radial_integrals_.zero();
             }
 
-            occupation_matrix_ = sddk::mdarray<double_complex, 4>(16, 16, 2, 2);
+            occupation_matrix_ = sddk::mdarray<std::complex<double>, 4>(16, 16, 2, 2);
 
-            uj_correction_matrix_ = sddk::mdarray<double_complex, 4>(16, 16, 2, 2);
+            uj_correction_matrix_ = sddk::mdarray<std::complex<double>, 4>(16, 16, 2, 2);
         }
 
         if (!type().parameters().full_potential()) {
@@ -145,7 +143,7 @@ class Atom
      *        V_{\ell m}(r) & \ell > 0 \end{array} \right.
      *  \f]
      */
-    inline void generate_radial_integrals(sddk::device_t pu__, sddk::Communicator const& comm__)
+    inline void generate_radial_integrals(sddk::device_t pu__, mpi::Communicator const& comm__)
     {
         PROFILE("sirius::Atom::generate_radial_integrals");
 
@@ -346,19 +344,19 @@ class Atom
     }
 
     /// Return atom position in fractional coordinates.
-    inline vector3d<double> const& position() const
+    inline r3::vector<double> const& position() const
     {
         return position_;
     }
 
     /// Set atom position in fractional coordinates.
-    inline void set_position(vector3d<double> position__)
+    inline void set_position(r3::vector<double> position__)
     {
         position_ = position__;
     }
 
     /// Return vector field.
-    inline vector3d<double> vector_field() const
+    inline auto vector_field() const
     {
         return vector_field_;
     }
@@ -387,7 +385,7 @@ class Atom
         }
     }
 
-    inline void sync_radial_integrals(sddk::Communicator const& comm__, int const rank__)
+    inline void sync_radial_integrals(mpi::Communicator const& comm__, int const rank__)
     {
         comm__.bcast(h_radial_integrals_.at(sddk::memory_t::host), (int)h_radial_integrals_.size(), rank__);
         if (type().parameters().num_mag_dims()) {
@@ -395,7 +393,7 @@ class Atom
         }
     }
 
-    inline void sync_occupation_matrix(sddk::Communicator const& comm__, int const rank__)
+    inline void sync_occupation_matrix(mpi::Communicator const& comm__, int const rank__)
     {
         comm__.bcast(occupation_matrix_.at(sddk::memory_t::host), (int)occupation_matrix_.size(), rank__);
     }
@@ -429,10 +427,10 @@ class Atom
      *  \f]
      */
     template <spin_block_t sblock>
-    inline double_complex
-    radial_integrals_sum_L3(int idxrf1__, int idxrf2__, std::vector<gaunt_L3<double_complex>> const& gnt__) const
+    inline std::complex<double>
+    radial_integrals_sum_L3(int idxrf1__, int idxrf2__, std::vector<gaunt_L3<std::complex<double>>> const& gnt__) const
     {
-        double_complex zsum(0, 0);
+        std::complex<double> zsum(0, 0);
 
         for (size_t i = 0; i < gnt__.size(); i++) {
             switch (sblock) {
@@ -455,13 +453,13 @@ class Atom
                 }
                 case spin_block_t::ud: {
                     /* Bx - i By */
-                    zsum += gnt__[i].coef * double_complex(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
+                    zsum += gnt__[i].coef * std::complex<double>(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
                                                            -b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 2));
                     break;
                 }
                 case spin_block_t::du: {
                     /* Bx + i By */
-                    zsum += gnt__[i].coef * double_complex(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
+                    zsum += gnt__[i].coef * std::complex<double>(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
                                                            b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 2));
                     break;
                 }
@@ -510,21 +508,21 @@ class Atom
         return type_.mt_lo_basis_size();
     }
 
-    inline void set_occupation_matrix(const double_complex* source)
+    inline void set_occupation_matrix(const std::complex<double>* source)
     {
-        std::memcpy(occupation_matrix_.at(sddk::memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(double_complex));
+        std::memcpy(occupation_matrix_.at(sddk::memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
         apply_uj_correction_ = false;
     }
 
-    inline void get_occupation_matrix(double_complex* destination)
+    inline void get_occupation_matrix(std::complex<double>* destination)
     {
-        std::memcpy(destination, occupation_matrix_.at(sddk::memory_t::host), 16 * 16 * 2 * 2 * sizeof(double_complex));
+        std::memcpy(destination, occupation_matrix_.at(sddk::memory_t::host), 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
     }
 
-    inline void set_uj_correction_matrix(const int l, const double_complex* source)
+    inline void set_uj_correction_matrix(const int l, const std::complex<double>* source)
     {
         uj_correction_l_ = l;
-        memcpy(uj_correction_matrix_.at(sddk::memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(double_complex));
+        std::memcpy(uj_correction_matrix_.at(sddk::memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
         apply_uj_correction_ = true;
     }
 
