@@ -55,7 +55,7 @@ Atom_symmetry_class::Atom_symmetry_class(int id__, Atom_type const& atom_type__)
         o1_radial_integrals_.zero();
     }
 
-    /* copy descriptors because enu is defferent between atom classes */
+    /* copy descriptors because enu is different between atom classes */
     aw_descriptors_.resize(atom_type_.num_aw_descriptors());
     for (int i = 0; i < num_aw_descriptors(); i++) {
         aw_descriptors_[i] = atom_type_.aw_descriptor(i);
@@ -221,7 +221,7 @@ Atom_symmetry_class::generate_lo_radial_functions(relativity_t rel__)
         double b[]    = {0, 0, 0};
         b[num_rs - 1] = 1.0;
 
-        int info = sddk::linalg(sddk::linalg_t::lapack).gesv(num_rs, 1, &a[0][0], 3, b, 3);
+        int info = la::wrap(la::lib_t::lapack).gesv(num_rs, 1, &a[0][0], 3, b, 3);
 
         if (info) {
             std::stringstream s;
@@ -353,7 +353,7 @@ Atom_symmetry_class::check_lo_linear_independence(double tol__)
     int nmtp = atom_type_.num_mt_points();
 
     Spline<double>  s(atom_type_.radial_grid());
-    sddk::dmatrix<double> loprod(num_lo_descriptors(), num_lo_descriptors());
+    la::dmatrix<double> loprod(num_lo_descriptors(), num_lo_descriptors());
     loprod.zero();
     for (int idxlo1 = 0; idxlo1 < num_lo_descriptors(); idxlo1++) {
 
@@ -376,10 +376,10 @@ Atom_symmetry_class::check_lo_linear_independence(double tol__)
     sddk::mdarray<double, 2> ovlp(num_lo_descriptors(), num_lo_descriptors());
     loprod >> ovlp;
 
-    auto stdevp = Eigensolver_factory("lapack");
+    auto stdevp = la::Eigensolver_factory("lapack");
 
     std::vector<double> loprod_eval(num_lo_descriptors());
-    sddk::dmatrix<double> loprod_evec(num_lo_descriptors(), num_lo_descriptors());
+    la::dmatrix<double> loprod_evec(num_lo_descriptors(), num_lo_descriptors());
 
     stdevp->solve(num_lo_descriptors(), loprod, &loprod_eval[0], loprod_evec);
 
@@ -415,8 +415,8 @@ Atom_symmetry_class::check_lo_linear_independence(double tol__)
         }
 
         std::vector<double> eval(ilo.size());
-        sddk::dmatrix<double> evec(static_cast<int>(ilo.size()), static_cast<int>(ilo.size()));
-        sddk::dmatrix<double> tmp(static_cast<int>(ilo.size()), static_cast<int>(ilo.size()));
+        la::dmatrix<double> evec(static_cast<int>(ilo.size()), static_cast<int>(ilo.size()));
+        la::dmatrix<double> tmp(static_cast<int>(ilo.size()), static_cast<int>(ilo.size()));
         for (int j1 = 0; j1 < (int)ilo.size(); j1++) {
             for (int j2 = 0; j2 < (int)ilo.size(); j2++) {
                 tmp(j1, j2) = ovlp(ilo[j1], ilo[j2]);
@@ -576,7 +576,7 @@ Atom_symmetry_class::generate_radial_functions(relativity_t rel__)
 }
 
 void
-Atom_symmetry_class::sync_radial_functions(sddk::Communicator const& comm__, int const rank__)
+Atom_symmetry_class::sync_radial_functions(mpi::Communicator const& comm__, int const rank__)
 {
     /* don't broadcast Hamiltonian radial functions, because they are used locally */
     int size = (int)(radial_functions_.size(0) * radial_functions_.size(1));
@@ -586,7 +586,7 @@ Atom_symmetry_class::sync_radial_functions(sddk::Communicator const& comm__, int
 }
 
 void
-Atom_symmetry_class::sync_radial_integrals(sddk::Communicator const& comm__, int const rank__)
+Atom_symmetry_class::sync_radial_integrals(mpi::Communicator const& comm__, int const rank__)
 {
     comm__.bcast(h_spherical_integrals_.at(sddk::memory_t::host), (int)h_spherical_integrals_.size(), rank__);
     comm__.bcast(o_radial_integrals_.at(sddk::memory_t::host), (int)o_radial_integrals_.size(), rank__);
@@ -597,7 +597,7 @@ Atom_symmetry_class::sync_radial_integrals(sddk::Communicator const& comm__, int
 }
 
 void
-Atom_symmetry_class::sync_core_charge_density(sddk::Communicator const& comm__, int const rank__)
+Atom_symmetry_class::sync_core_charge_density(mpi::Communicator const& comm__, int const rank__)
 {
     RTE_ASSERT(ae_core_charge_density_.size() != 0);
 
@@ -732,7 +732,7 @@ Atom_symmetry_class::generate_radial_integrals(relativity_t rel__)
 }
 
 void
-Atom_symmetry_class::write_enu(sddk::pstdout& pout) const
+Atom_symmetry_class::write_enu(mpi::pstdout& pout) const
 {
     pout << "Atom : " << atom_type_.symbol() << ", class id : " << id_ << std::endl;
     pout << "augmented waves" << std::endl;
