@@ -41,6 +41,45 @@ inline void throw_impl(const char* func__, const char* file__, int line__, std::
     throw_impl(func__, file__, line__, msg.str(), pmsg);
 }
 
+class ostream : public std::ostringstream
+{
+  private:
+    std::ostream* out_{nullptr};
+    std::string prefix_;
+  public:
+    ostream()
+    {
+    }
+    ostream(std::ostream& out__, std::string prefix__)
+        : out_(&out__)
+        , prefix_(prefix__)
+    {
+    }
+    ostream(ostream&& src__)
+      : std::ostringstream(std::move(src__))
+    {
+        out_ = src__.out_;
+        src__.out_ = nullptr;
+        prefix_ = src__.prefix_;
+    }
+    ~ostream()
+    {
+        if (out_) {
+            auto strings = rte::split(this->str());
+            for (size_t i = 0; i < strings.size(); i++) {
+                if (!(i == strings.size() - 1 && strings[i].size() == 0)) {
+                    (*out_) << "[" << prefix_ << "] " << strings[i];
+                }
+                if (i != strings.size() - 1) {
+                    (*out_) << std::endl;
+                }
+            }
+        }
+    }
+};
+
+#define FILE_LINE std::string(__FILE__) + ":" + std::to_string(__LINE__)
+
 #define RTE_THROW(...) \
 {\
     ::rte::throw_impl(__func__, __FILE__, __LINE__, __VA_ARGS__);\
@@ -60,32 +99,7 @@ inline void throw_impl(const char* func__, const char* file__, int line__, std::
 }
 #endif
 
-class rte_ostream : public std::ostringstream
-{
-  private:
-    std::ostream& out_;
-    std::string prefix_;
-  public:
-    rte_ostream(std::ostream& out__, std::string prefix__)
-        : out_(out__)
-        , prefix_(prefix__)
-    {
-    }
-    ~rte_ostream()
-    {
-        auto strings = rte::split(this->str());
-        for (size_t i = 0; i < strings.size(); i++) {
-            if (!(i == strings.size() - 1 && strings[i].size() == 0)) {
-                out_ << "[" << prefix_ << "] " << strings[i];
-            }
-            if (i != strings.size() - 1) {
-                out_ << std::endl;
-            }
-        }
-    }
-};
-
-#define RTE_OUT(_out) rte::rte_ostream(_out, std::string(__func__))
+#define RTE_OUT(_out) rte::ostream(_out, std::string(__func__))
 
 }
 
